Package: RaggedExperiment
Title: Representation of Sparse Experiments and Assays Across Samples
Version: 1.34.0
Authors@R: c(
    person("Martin", "Morgan", , "martin.morgan@roswellpark.org", "aut"),
    person("Marcel", "Ramos", , "marcel.ramos@sph.cuny.edu",
        c("aut", "cre"), c(ORCID = "0000-0002-3242-0582")),
    person("Lydia", "King", , "L.King18@nuigalway.ie", "ctb")
    )
Description: This package provides a flexible representation of copy
    number, mutation, and other data that fit into the ragged array
    schema for genomic location data. The basic representation of such
    data provides a rectangular flat table interface to the user with
    range information in the rows and samples/specimen in the columns.
    The RaggedExperiment class derives from a GRangesList representation
    and provides a semblance of a rectangular dataset.
License: Artistic-2.0
biocViews: Infrastructure, DataRepresentation
BugReports: https://github.com/Bioconductor/RaggedExperiment/issues
URL: https://bioconductor.github.io/RaggedExperiment,
        https://bioconductor.org/packages/RaggedExperiment
VignetteBuilder: knitr
Depends: R (>= 4.5.0), GenomicRanges (>= 1.61.1)
Imports: BiocBaseUtils, BiocGenerics, Seqinfo, IRanges, Matrix,
        MatrixGenerics, methods, S4Vectors, stats, SummarizedExperiment
        (>= 1.39.1), utils
Suggests: BiocStyle, knitr, rmarkdown, testthat, GenomeInfoDb,
        MultiAssayExperiment
RoxygenNote: 7.3.2
Encoding: UTF-8
Date: 2024-10-17
git_url: https://git.bioconductor.org/packages/RaggedExperiment
git_branch: RELEASE_3_22
git_last_commit: 2cf1b41
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:54:50 UTC; biocbuild
Author: Martin Morgan [aut],
  Marcel Ramos [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3242-0582>),
  Lydia King [ctb]
Maintainer: Marcel Ramos <marcel.ramos@sph.cuny.edu>

# Generated by roxygen2: do not edit by hand

S3method(.DollarNames,RaggedExperiment)
export(RaggedExperiment)
export(compactAssay)
export(compactSummarizedExperiment)
export(disjoinAssay)
export(disjoinSummarizedExperiment)
export(qreduceAssay)
export(qreduceSummarizedExperiment)
export(sparseAssay)
export(sparseSummarizedExperiment)
exportClasses(RaggedExperiment)
exportMethods("$")
exportMethods("[")
exportMethods("colData<-")
exportMethods("dimnames<-")
exportMethods("mcols<-")
exportMethods("rowData<-")
exportMethods("rowRanges<-")
exportMethods("seqinfo<-")
exportMethods(as.data.frame)
exportMethods(as.list)
exportMethods(assay)
exportMethods(assayNames)
exportMethods(assays)
exportMethods(coerce)
exportMethods(colData)
exportMethods(dim)
exportMethods(dimnames)
exportMethods(length)
exportMethods(mcols)
exportMethods(overlapsAny)
exportMethods(rowData)
exportMethods(rowRanges)
exportMethods(seqinfo)
exportMethods(show)
exportMethods(subset)
exportMethods(subsetByOverlaps)
import(BiocGenerics)
import(GenomicRanges)
import(IRanges)
import(S4Vectors)
import(SummarizedExperiment)
import(methods)
importClassesFrom(GenomicRanges,GRangesList)
importClassesFrom(S4Vectors,Annotated)
importFrom(BiocGenerics,relist)
importFrom(GenomicRanges,GRanges)
importFrom(MatrixGenerics,rowRanges)
importFrom(S4Vectors,mcols)
importFrom(Seqinfo,"seqinfo<-")
importFrom(Seqinfo,seqinfo)
importFrom(SummarizedExperiment,"rowRanges<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,rowData)
importFrom(stats,setNames)
importFrom(utils,.DollarNames)

#' @import methods S4Vectors BiocGenerics IRanges
#' @import S4Vectors GenomicRanges SummarizedExperiment
#' @importClassesFrom S4Vectors Annotated
#' @importClassesFrom GenomicRanges GRangesList
#' @importFrom S4Vectors mcols
#' @importFrom BiocGenerics relist
#' @importFrom Seqinfo seqinfo seqinfo<-
#' @importFrom MatrixGenerics rowRanges
#' @importFrom SummarizedExperiment rowRanges<-
#' @importFrom stats setNames
NULL

#' @exportClass RaggedExperiment
.RaggedExperiment <- setClass(
    "RaggedExperiment",
    slots = c(
        assays = "GRangesList",
        rowidx = "integer",
        colidx = "integer"
    ),
    contains = "Annotated"
)

.valid.RaggedExperiment.idx <- function(x) {
    txt <- character()
    if (!all(.rowidx(x) %in% seq_along(.rowRanges(x))))
        txt <- c(txt, "(internal) '.rowidx()' outside .rowRanges()")
    if (!all(.colidx(x) %in% seq_along(.assays(x))))
        txt <- c(txt, "(internal) '.colidx()' outside .assays()")
    if (length(txt)) txt else NULL
}

.valid.RaggedExperiment <- function(x) {
    c(.valid.RaggedExperiment.idx(x))
}

setValidity2("RaggedExperiment", .valid.RaggedExperiment)

#' @name RaggedExperiment-class
#'
#' @aliases coerce,RaggedExperiment,GRangesList-method
#' coerce,GRangesList,RaggedExperiment-method RaggedExperiment-class
#' class:RaggedExperiment RaggedExperiment
#'
#' @title RaggedExperiment objects
#'
#' @description The \code{RaggedExperiment} class is a container for
#'     storing range-based data, including but not limited to copy
#'     number data, and mutation data. It can store a collection of
#'     \code{GRanges} objects, as it is derived from the
#'     \code{GenomicRangesList}.
#'
#' @section Constructors:
#'
#' \code{RaggedExperiment(..., colData=DataFrame())}: Creates a
#' RaggedExperiment object using multiple \code{GRanges} objects or a list
#' of \code{GRanges} objects.  Additional column data may be provided
#' as a \code{DataFrame} object.
#'
#' @section Accessors:
#'
#' In the following, 'x' represents a \code{RaggedExperiment} object:
#'
#'     \code{rowRanges(x)}:
#'
#'     Get the ranged data. Value is a \code{GenomicRanges} object.
#'
#'     \code{assays(x)}:
#'
#'     Get the assays. Value is a \code{\link[S4Vectors]{SimpleList}}.
#'
#'     \code{assay(x, i)}:
#'
#'     An alternative to \code{assays(x)[[i]]} to get the \emph{i}th
#'     (default first) assay element.
#'
#'     \code{mcols(x), mcols(x) <- value}:
#'
#'     Get or set the metadata columns. For \code{RaggedExperiment}, the
#'     columns correspond to the assay \emph{i}th elements.
#'
#'     \code{rowData(x), rowData(x) <- value}:
#'
#'     Get or set the row data. Value is a \code{\link[S4Vectors]{DataFrame}}
#'     object. Also corresponds to the \code{mcols} data.
#'
#'     \strong{\emph{Note}} for advanced users and developers. Both
#'     \code{mcols} and \code{rowData} setters may reduce the size of the
#'     internal \code{RaggedExperiment} data representation. Particularly after
#'     subsetting, the internal row index is modified and such setter
#'     operations will use the index to subset the data and reduce the
#'     "rows" of the internal data representation.
#'
#' @section Subsetting:
#'
#'    \code{x[i, j]}:
#'    Get ranges or elements (\code{i} and \code{j}, respectively) with
#'    optional metadata columns where \code{i} or \code{j} can be missing,
#'    an NA-free logical, numeric, or character vector.
#'
#' @section Coercion:
#'
#' In the following, 'object' represents a \code{RaggedExperiment} object:
#'
#' \code{as(object, "GRangesList")}:
#'
#' Creates a \link[GenomicRanges]{GRangesList} object from a
#' \code{RaggedExperiment}.
#'
#' \code{as(from, "RaggedExperiment")}:
#'
#' Creates a \code{RaggedExperiment} object from a
#' \link[GenomicRanges]{GRangesList}, or \link[GenomicRanges]{GRanges} object.
#'
#' @param ... Constructor: GRanges, list of GRanges, or GRangesList OR
#'     assay: Additional arguments for assay. See details for more information.
#'
#' @param colData A \code{\link[S4Vectors]{DataFrame}} describing samples.
#'   Length of rowRanges must equal the number of rows in colData
#'
#' @param metadata A \code{list} to include in the metadata. Any metadata
#'   included in the input objects are lost.
#'
#' @return constructor returns a \code{RaggedExperiment} object
#'
#' @example inst/scripts/RaggedExperiment-Ex.R
#'
#' @export
RaggedExperiment <- function(..., colData=DataFrame(), metadata = list()) {
    inputs <- list(...)
    if (length(inputs) == 1L && is(inputs[[1L]], "GenomicRangesList")) {
        GRList <- inputs[[1L]]
        rowRanges <- relist(unlist(unname(GRList)), PartitioningByEnd(GRList))
        if (isEmpty(colData))
            colData <- mcols(inputs[[1L]])
    } else if (identical(length(inputs), 1L) && is(inputs[[1L]], "list"))
        rowRanges <- GRangesList(inputs[[1L]])
    else
        rowRanges <- GRangesList(inputs)
    if (missing(colData) && 0L != length(rowRanges)) {
        nms <- names(rowRanges)
        colData <- DataFrame(x = seq_along(rowRanges), row.names = nms)[, FALSE]
    } else if (!missing(colData)) {
        if (!is(colData, "DataFrame"))
            colData <- as(colData, "DataFrame")
        if (length(rowRanges) != nrow(colData))
            stop("length() of 'rowRanges' is not equal to nrow() of 'colData'")
        if (!is.null(names(rowRanges))) {
            if (is.null(rownames(colData))) {
                rownames(colData) <- names(rowRanges)
            } else if (!setequal(names(rowRanges), rownames(colData))) {
                stop("'names(rowRanges)', 'rownames(colData)' are unequal")
            } else {
                colData <- colData[names(rowRanges), , drop=FALSE]
            }
        } else {
            names(rowRanges) <- rownames(colData)
        }
    }
    ans_colnames <- rownames(colData)
    if (!isEmpty(mcols(rowRanges)))
        warning("'mcols(rowRanges)' removed")
    mcols(rowRanges) <- colData
    .RaggedExperiment(
        assays = rowRanges,
        rowidx = seq_len(sum(lengths(rowRanges))),
        colidx = seq_along(rowRanges),
        metadata = metadata
    )
}

.assays <- function(x) {
    x@assays
}

.rowidx <- function(x) {
    x@rowidx
}

.colidx <- function(x) {
    x@colidx
}

.dim <- function(x) {
    assays <- .assays(x)
    c(sum(lengths(assays)), length(assays))
}

.dimnames <- function(x) {
    assays <- .assays(x)
    list(names(unlist(assays, use.names=FALSE)), names(assays))
}

.rowRanges <- function(x) {
    ranges <- unlist(.assays(x), use.names = FALSE)
    mcols(ranges) <- NULL
    ranges
}

.mcols <- function(x) {
    ranges <- unlist(.assays(x), use.names = FALSE)
    mcols(ranges)
}

.uranges <- function(x) {
    assays <- .assays(x)
    unlist(assays, use.names = FALSE)[.rowidx(x), ]
}

#' @describeIn RaggedExperiment seqinfo accessor
#' @exportMethod seqinfo
setMethod("seqinfo", "RaggedExperiment", function(x) {
    seqinfo(.assays(x))
})

#' @describeIn RaggedExperiment Replace seqinfo metadata of the ranges
#' @exportMethod seqinfo<-
#' @inheritParams Seqinfo::`seqinfo<-`
setReplaceMethod("seqinfo", "RaggedExperiment",
    function(x, new2old=NULL, pruning.mode=c("error", "coarse", "fine", "tidy"),
        value) {
        newAssay <- `seqinfo<-`(.assays(x), new2old = new2old,
            pruning.mode = pruning.mode, value = value)
        BiocBaseUtils::setSlots(x, assays = newAssay)
})

#' @describeIn RaggedExperiment rowRanges accessor
#' @return 'rowRanges' returns a \code{\link[GenomicRanges]{GRanges}} object
#'     summarizing ranges corresponding to \code{assay()} rows.
#' @exportMethod rowRanges
setMethod("rowRanges", "RaggedExperiment", function(x, ...) {
    .rowRanges(x)[.rowidx(x)]
})

#' @describeIn RaggedExperiment rowRanges replacement
#' @return 'rowRanges<-' returns a \code{\link{RaggedExperiment}} object
#'     with replaced ranges
#' @exportMethod rowRanges<-
setReplaceMethod("rowRanges", c("RaggedExperiment", "GRanges"),
    function(x, ..., value) {
        if (isEmpty(mcols(value)))
            mcols(value) <- .mcols(x)
        value <- relist(value, .assays(x))
        BiocBaseUtils::setSlots(x, assays = value, check = FALSE)
    }
)

#' @describeIn RaggedExperiment get the metadata columns of the ranges,
#'     rectangular representation of the 'assays'
#' @return 'mcols' returns a \code{\link[S4Vectors]{DataFrame}} object
#'     of the metadata columns
#' @param use.names (logical default FALSE) whether to propagate rownames from
#' the object to rownames of metadata \code{DataFrame}
#'
#' @exportMethod mcols
setMethod("mcols", "RaggedExperiment", function(x, use.names = FALSE, ...) {
    ranges <- .uranges(x)
    mcols(ranges, use.names = use.names)
})

#' @describeIn RaggedExperiment set the metadata columns of the ranges
#'     corresponding to the assays
#' @param value \itemize{
#'     \item{dimnames}: A \code{list} of dimension names
#'     \item{mcols}: A \code{\link[S4Vectors]{DataFrame}} representing the
#'     assays
#'     }
#'
#' @exportMethod mcols<-
setReplaceMethod("mcols", "RaggedExperiment", function(x, ..., value) {
    ranges <- .uranges(x)
    assays <- .assays(x)

    mcols(ranges) <- value
    assays <- relist(ranges, assays)

    BiocBaseUtils::setSlots(x, assays = assays, check = FALSE)
})

#' @describeIn RaggedExperiment get the rowData or metadata for the ranges
#' @exportMethod rowData
setMethod("rowData", "RaggedExperiment",
    function(x, ...) mcols(x, ...)
)

#' @describeIn RaggedExperiment set the rowData or metadata for the ranges
#' @exportMethod rowData<-
setReplaceMethod("rowData", "RaggedExperiment",
    function(x, ..., value) `mcols<-`(x, ..., value=value)
)

#' @describeIn RaggedExperiment get dimensions (number of sample-specific row
#'     ranges by number of samples)
#' @exportMethod dim
setMethod("dim", "RaggedExperiment", function(x) {
    c(length(.rowidx(x)), length(.colidx(x)))
})

#' @describeIn RaggedExperiment get row (sample-specific) range names
#'     and sample names
#' @exportMethod dimnames
setMethod("dimnames", "RaggedExperiment", function(x) {
    dimnames <- .dimnames(x)
    list(dimnames[[1]][.rowidx(x)], dimnames[[2]][.colidx(x)])
})

#' @describeIn RaggedExperiment set row (sample-specific) range names
#'     and sample names
#' @exportMethod dimnames<-
setReplaceMethod("dimnames", c("RaggedExperiment", "list"),
    function(x, value)
{
    new_rownames <- value[[1L]]
    new_colnames <- value[[2L]]
    x_assays <- .assays(x)
    if (is.null(new_colnames))
        names(x_assays) <- NULL
    else
        names(x_assays)[.colidx(x)] <- new_colnames
    unlisted_assays <- unlist(x_assays, use.names = FALSE)
    if (is.null(new_rownames))
        names(unlisted_assays) <- NULL
    else
        names(unlisted_assays)[.rowidx(x)] <- new_rownames

    new_assays <- relist(unlisted_assays, x_assays)
    mcols(new_assays) <- mcols(x_assays)
    BiocBaseUtils::setSlots(x, assays = new_assays, check = FALSE)
})

#' @describeIn RaggedExperiment set row range names and sample names to NULL
setReplaceMethod("dimnames", c("RaggedExperiment", "ANY"),
    function(x, value)
{
    if (!is.null(value))
        stop("'dimnames' must be a list")

    x_assays <- .assays(x)
    names(x_assays) <- NULL
    unlisted_assays <- unlist(x_assays, use.names=FALSE)
    names(unlisted_assays) <- NULL
    new_assays <- relist(unlisted_assays, x_assays)
    mcols(new_assays) <- mcols(x_assays)
    BiocBaseUtils::setSlots(x, assays = new_assays, check = FALSE)
})

#' @describeIn RaggedExperiment get the length of row vectors in the object,
#'   similar to \link[SummarizedExperiment]{SummarizedExperiment}
#' @exportMethod length
setMethod("length", "RaggedExperiment", function(x) {
    length(.rowidx(x))
})

#' @describeIn RaggedExperiment get column data
#' @exportMethod colData
setMethod("colData", "RaggedExperiment", function(x, ...) {
    mcols(.assays(x), use.names=TRUE)[.colidx(x), , drop=FALSE]
})

#' @describeIn RaggedExperiment change the colData
#' @exportMethod colData<-
setReplaceMethod("colData", c("RaggedExperiment", "DataFrame"),
    function(x, value) {
        ranges <- .assays(x)
        mcols(ranges) <- value
        x@assays <- ranges
        x
    })

#' @describeIn RaggedExperiment assay missing method uses first metadata column
#' @exportMethod assay
setMethod("assay", c("RaggedExperiment", "missing"),
    function(x, i, withDimnames = TRUE, ...) {
        assay(x, i=1L, withDimnames = withDimnames, ...)
    }
)

#' @describeIn RaggedExperiment assay numeric method.
#' @param i logical(1), integer(1), or character(1) indicating the
#'     assay to be reported. For \code{[}, \code{i} can be any
#'     supported \code{Vector} object, e.g., \code{GRanges}.
#' @exportMethod assay
setMethod("assay", c("RaggedExperiment", "ANY"),
    function(x, i, withDimnames = TRUE, ...)
{
    sparseAssay(x, i, withDimnames = withDimnames, ...)
})

#' @describeIn RaggedExperiment assays
#' @param x A RaggedExperiment object.
#' @param withDimnames logical (default TRUE) whether to use dimension names
#' in the resulting object
#' @return 'assays' returns a \code{\link[S4Vectors]{SimpleList}}
#' @exportMethod assays
setMethod("assays", "RaggedExperiment", function(x, withDimnames = TRUE, ...) {
    nms <- names(.mcols(x))
    lst <- lapply(nms, function(index, obj) {
            assay(obj, index, withDimnames = withDimnames, ...)
        }, obj = x)
    SimpleList(setNames(lst, nms))
})

#' @describeIn RaggedExperiment names in each assay
#' @exportMethod assayNames
setMethod("assayNames", "RaggedExperiment", function(x, ...) {
    names(.mcols(x))
})

#' @describeIn RaggedExperiment show method
#' @param object A RaggedExperiment object.
#' @exportMethod show
setMethod("show", "RaggedExperiment", function(object) {
    scat <- function(fmt, vals = character(), exdent = 2, ...) {
        vals <- ifelse(nzchar(vals), vals, "''")
        lbls <- paste(BiocBaseUtils::selectSome(vals), collapse = " ")
        txt <- sprintf(fmt, length(vals), lbls)
        cat(strwrap(txt, exdent = exdent, ...), sep = "\n")
    }
    cat("class:", class(object), "\n")
    cat("dim:", dim(object), "\n")

    # expt <- names(metadata(object))
    # if (is.null(expt))
    #     expt <- character(length(metadata(object)))
    # scat("metadata(%d): %s\n", expt)

    nms <- assayNames(object)
    if (is.null(nms))
        nms <- character(length(assays(object, withDimnames = FALSE)))
    scat("assays(%d): %s\n", nms)

    dimnames <- dimnames(object)
    dlen <- vapply(dimnames, length, integer(1L))
    if (dlen[[1]])
        scat("rownames(%d): %s\n", dimnames[[1]])
    else cat("rownames: NULL\n")
    if (dlen[[2]])
        scat("colnames(%d): %s\n", dimnames[[2]])
    else cat("colnames: NULL\n")

    scat("colData names(%d): %s\n", names(colData(object)))
})

setAs("RaggedExperiment", "GRangesList", function(from) {
    .assays(from)[.colidx(from)]
})

setAs("GRangesList", "RaggedExperiment", function(from) {
    RaggedExperiment(from)
})


#' @describeIn RaggedExperiment Allow extraction of metadata columns as a plain
#'   \code{list}
#'
#' @export
setMethod("as.list", "RaggedExperiment", function(x, ...) {
    lapply(as(x, "GRangesList"), mcols)
})

#' @describeIn RaggedExperiment Allow conversion to plain \code{data.frame}
#'
#' @inheritParams base::as.data.frame
#'
#' @export
setMethod("as.data.frame", "RaggedExperiment",
    function(x, row.names = NULL, optional = FALSE, ...) {
        as.data.frame(
            as(x, "GRangesList"),
            row.names = row.names,
            optional = optional,
            ...
        )
    }
)

#' @importFrom utils .DollarNames
#' @export
.DollarNames.RaggedExperiment <- function(x, pattern = "")
    grep(pattern, names(colData(x)), value = TRUE)

#' @aliases $,RaggedExperiment-method
#' @describeIn RaggedExperiment Easily access the \code{colData} columns with
#'   the dollar sign operator
#' @inheritParams base::`$`
#' @exportMethod $
setMethod("$", "RaggedExperiment", function(x, name) {
    colData(x)[[name]]
})

context("RaggedExperiment-class")

test_that("constructors work for basic cases", {
    expect_true(validObject(RaggedExperiment()))
    expect_true(validObject(RaggedExperiment(GRanges(), GRanges())))

    lgr1 <- list(GRanges(), GRanges())
    expect_true(validObject(RaggedExperiment(lgr1)))

    lgr2 <- as(list(GRanges(), GRanges()), "GRangesList")
    expect_true(validObject(RaggedExperiment(lgr2)))

    lgr3 <- as(list(GRanges(), GRanges()), "CompressedGRangesList")
    expect_true(validObject(RaggedExperiment(lgr3)))

    grl <- GRangesList(GRanges(), GRanges())
    expect_true(validObject(RaggedExperiment(grl)))
})

test_that("colData construction works", {
    expect_true(validObject(RaggedExperiment(colData=DataFrame())))
    expect_true(validObject(RaggedExperiment(colData=data.frame())))

    gr <- GRanges()
    colData <- DataFrame(x=1)
    expect_true(validObject(RaggedExperiment(gr, colData=colData)))
    expect_error(RaggedExperiment(gr, gr, colData=colData))

    colData <- DataFrame(x=1:2)
    expect_true(validObject(RaggedExperiment(gr, gr, colData=colData)))
    ## names of assays should match rownames of colData

    # consistent names(assays) / rownames(colData)
    grl <- GRangesList(A=GRanges(), B=GRanges())
    colData <- DataFrame(x=1:2, row.names=LETTERS[1:2])
    expect_true(validObject(RaggedExperiment(grl, colData=colData)))

    # missing names(assays) -- OK, remove rownames(colData)
    grl <- GRangesList(GRanges(), GRanges())
    colData <- DataFrame(x=1:2, row.names=LETTERS[1:2])
    re <- RaggedExperiment(grl, colData=colData)
    expect_true(validObject(re))
    expect_identical(rownames(colData(re)), rownames(colData))

    # consistent out-of-order names(assays) / rownames(colData)
    grl <- GRangesList(B=GRanges(), A=GRanges())
    colData <- DataFrame(x=1:2, row.names=LETTERS[1:2])
    re <- RaggedExperiment(grl, colData=colData)
    expect_true(validObject(re))
    expect_identical(colnames(re), names(grl))

    # inconsistent names(assays) / rownames(colData)
    grl <- GRangesList(A=GRanges(), B=GRanges())
    colData <- DataFrame(x=1:2, row.names=LETTERS[2:3])
    expect_error(RaggedExperiment(grl, colData=colData))
})

test_that("colData() works", {
    sample1 <- GRanges(c("chr1:1-10", "chr1:11-18"), score = 1:2)
    sample2 <- GRanges(c("chr1:1-10", "chr2:11-18"), score = 3:4)
    re <- RaggedExperiment(sample1, sample2)

    colData <- DataFrame(x=1:2)[, FALSE]
    expect_identical(colData(re), colData)

    colData <- DataFrame(x=1:2)
    re <- RaggedExperiment(sample1, sample2, colData=colData)
    expect_identical(colData(re), colData)

    colData <- DataFrame(x=1:2, row.names=LETTERS[1:2])
    re <- RaggedExperiment(list(A=sample1, B=sample2), colData=colData)
    expect_identical(colData(re), colData)
})

test_that("rowData() works", {
    grl <- GRangesList(
        sample1 = GRanges( c("chr1:1-10", "chr2:15-18", "chr2:25-34") ),
        sample2 = GRanges( c("chr1:1-10", "chr2:11-18" , "chr2:25-36") ),
        sample3 = GRanges( c("chr1:2-11", "chr2:14-18", "chr2:26-36") ),
        sample4 = GRanges( c("chr1:1-12", "chr2:18-35" ) ),
        sample5 = GRanges( c("chr1:1-12", "chr2:11-17" , "chr2:26-34") ) ,
        sample6 = GRanges( c("chr1:1-12", "chr2:12-18" , "chr2:25-35") )
    )

    ra <- RaggedExperiment(grl)
    rowannote <- DataFrame(labels = letters[1:17], numbers = 1:17)
    mcols(ra) <- rowannote
    expect_identical(rowannote, rowData(ra))
    expect_identical(rowannote, mcols(ra))

    expect_equal(length(assays(ra)), ncol(rowannote))
})

test_that("rowRanges() works", {
    re <- RaggedExperiment()
    expect_identical(rowRanges(re), GRanges())

    rowRanges(re) <- GRanges()
    expect_identical(rowRanges(re), GRanges())

    re <- RaggedExperiment(GRanges(), GRanges())
    expect_identical(rowRanges(re), GRanges())

    rng <- GRanges(c("A:1-5", "B:1-5"))
    rng2 <- rng + 1

    re <- RaggedExperiment(rng)
    expect_identical(dim(re), c(2L, 1L))
    expect_identical(rowRanges(re), rng)

    rowRanges(re) <- rng2
    expect_identical(rowRanges(re), rng2)

    re <- RaggedExperiment(split(rng, letters[1:2]))
    expect_identical(dim(re), c(2L, 2L))
    expect_identical(rowRanges(re), rng)

    rng <- GRanges(c("A", "B"), IRanges(1, 5))
    re <- RaggedExperiment(rng)
    expect_identical(dim(re), c(2L, 1L))
    expect_identical(rowRanges(re), rng)

    re <- RaggedExperiment(split(rng, letters[1:2]))
    expect_identical(dim(re), c(2L, 2L))
    expect_identical(rowRanges(re), rng)

    ## subsetting
    rng <- GRanges("A", IRanges(1:4, width=5))
    re <- RaggedExperiment(rng)
    expect_identical(rowRanges(re[FALSE]), rng[FALSE])
    expect_identical(rowRanges(re[3:1]), rng[3:1])

    re <- RaggedExperiment(split(rng, rep(letters[1:2], each=2)))
    expect_identical(rowRanges(re[3:1]), rng[3:1])

    re <- RaggedExperiment(split(rng, letters[1:4]))
    expect_identical(rowRanges(re[c(3, 1)]), rng[c(3, 1)])
})

test_that("assay() selection works", {
    lgr <- list(
        A=GRanges(
            c(a="chr1:1-10", b="chr1:6-15"),
            score1 = 1:2,
            score2 = -(1:2)),
        B=GRanges(
            c(c="chr1:1-10", d="chr1:6-15"),
            score1 = 3:4,
            score2 = -(3:4)))
    colData <- DataFrame(x=1:2, id=LETTERS[1:2])
    re <- RaggedExperiment(lgr, colData=colData)
    m1 <- matrix(
        c(1L, 2L, NA, NA, NA, NA, 3L, 4L), 4,
        dimnames=list(letters[1:4], LETTERS[1:2]))
    m2 <- -m1

    expect_identical(assay(re), m1)
    expect_identical(assay(re, 1), m1)
    expect_identical(assay(re, 2), m2)

    expect_identical(names(assays(re)), names(mcols(lgr[[1]])))
    expect_identical(assay(re, "score1"), m1)
    expect_identical(assay(re, "score2"), m2)
})

test_that("assay() works", {
    sample1 <- GRanges(c("chr1:1-10", "chr1:11-18"), score = 1:2)
    sample2 <- GRanges(c("chr1:1-10", "chr2:11-18"), score = 3:4)
    re <- RaggedExperiment(sample1, sample2)

    m0 <- matrix(
        c(1L, 2L, NA, NA, NA, NA, 3L, 4L), ncol=2,
        dimnames=list(as.character(rowRanges(re)), NULL)
    )
    expect_identical(assay(re), m0)
    expect_identical(
        assay(re[FALSE, FALSE]), assay(re)[FALSE, FALSE, drop=FALSE]
    )

    expect_identical(assay(re[2,]), assay(re)[2,, drop=FALSE])
    expect_identical(assay(re[, 2]), assay(re)[,2, drop=FALSE])

    ridx <- c(2, 1, 3)
    expect_identical(assay(re[ridx,]), assay(re)[ridx,])

    sample3 <- GRanges(c(a = "chr1:1-10", b = "chr1:11-18"),
        score = factor(1:2))
    retst <- RaggedExperiment(sample3 = sample3)

    m1 <- matrix(
        as.character(1:2), ncol = 1L,
        dimnames = list(c("a", "b"), "sample3"))

    expect_identical(assay(retst), m1)
})

test_that("dimnames() and dimnames<-() work", {
    re <- RaggedExperiment()
    dimnames(re) = dimnames(re)
    expect_identical(dimnames(re), list(NULL, NULL))

    sample1 <- GRanges(c("chr1:1-10", "chr1:11-18"), score = 1:2)
    sample2 <- GRanges(c("chr1:1-10", "chr2:11-18"), score = 3:4)

    re <- RaggedExperiment(sample1, sample2)
    expect_identical(dimnames(re), list(NULL, NULL))

    nms <- list(letters[1:4], LETTERS[1:2])
    dimnames(re) <- nms
    expect_identical(dimnames(re), nms)

    rownames(re) <-  LETTERS[1:4]
    expect_identical(dimnames(re), list(LETTERS[1:4], LETTERS[1:2]))
    expect_identical(colnames(re), rownames(colData(re)))

    re <- RaggedExperiment(sample1, sample2)[4:1]
    rownames(re) <- letters[1:4]
    expect_identical(rownames(re), letters[1:4])

    re <- RaggedExperiment(sample1, sample2)
    dimnames(re) <- nms
    dimnames(re) <- NULL
    expect_identical(dimnames(re), list(NULL, NULL))

    re <- RaggedExperiment(sample1, sample2)
    expect_error(
        dimnames(re) <- letters[1:4]
    )
})

test_that("Dollar sign operator ($) works", {
    sample1 <- GRanges(c("chr1:1-10", "chr1:11-18"), score = 1:2)
    sample2 <- GRanges(c("chr1:1-10", "chr2:11-18"), score = 3:4)
    coldata <- DataFrame(type = c("A", "B"), id = 1:2)
    re <- RaggedExperiment(sample1, sample2, colData = coldata)
    expect_identical(re$id, c(1L, 2L))
    expect_identical(re$type, LETTERS[1:2])
})

context("RaggedExperiment-subset-methods")

test_that("[ subsetting works", {
    lgr <- list(
        A=GRanges(
            c(a="chr1:1-10", b="chr1:6-15"),
            score1 = 1:2,
            score2 = -(1:2)),
        B=GRanges(
            c(c="chr1:1-10", d="chr1:6-15"),
            score1 = 3:4,
            score2 = -(3:4)))
    colData <- DataFrame(x=1:2, id=LETTERS[1:2])
    re <- RaggedExperiment(lgr, colData=colData)

    expect_identical(re[], re)
    expect_identical(re[,], re)
    expect_identical(re[TRUE,], re)
    expect_identical(re[, TRUE], re)
    expect_identical(re[TRUE, TRUE], re)

    test <- re[1,]
    expect_identical(dim(test), c(1L, 2L))
    expect_identical(dimnames(test), list("a", c("A", "B")))
    expect_identical(assay(test), assay(re)[1,, drop=FALSE])

    test <- re[3:4,]
    expect_identical(dim(test), c(2L, 2L))
    expect_identical(dimnames(test), list(c("c", "d"), c("A", "B")))
    expect_identical(assay(test), assay(re)[3:4,, drop=FALSE])

    test <- re[2:3,]
    expect_identical(dim(test), c(2L, 2L))
    expect_identical(dimnames(test), list(c("b", "c"), c("A", "B")))
    expect_identical(assay(test), assay(re)[2:3,, drop=FALSE])

    test <- re[3:4, 2]
    expect_equal(test, RaggedExperiment(lgr[2], colData=colData[2,]))

    test <- re[4:1,][4:1,]
    expect_identical(test, re)

    test <- re[c(1, 3, 1),]
    expect_identical(dim(test), c(3L, 2L))
    expect_identical(dimnames(test), list(c("a", "c", "a"), c("A", "B")))
    expect_identical(assay(test), assay(re)[c(1, 3, 1),, drop=FALSE])

    test <- re[c(1, 3, 1),][2,]
    expect_identical(dim(test), c(1L, 2L))
    expect_identical(dimnames(test), list("c", c("A", "B")))
    expect_identical(assay(test), assay(re)[3,, drop=FALSE])

    test <- re[, 2]
    expect_identical(dim(test), c(4L, 1L))
    expect_identical(dimnames(test), list(rownames(re), "B"))
    expect_identical(assay(test), assay(re)[, 2, drop=FALSE])

    test <- re[,2:1]
    expect_identical(dim(test), c(4L, 2L))
    expect_identical(dimnames(test), list(rownames(re), colnames(re)[2:1]))
    expect_identical(assay(test), assay(re)[, 2:1, drop=FALSE])

    test <- re[, 2:1][, 2:1]
    expect_identical(test, re)

    test <- re[4:1, 2:1][4:1, 2:1]
    expect_identical(test, re)

    ## auto-dimnames
    lgr <- list(
        GRanges(c("chr1:1-10", "chr1:6-15"), score1 = 1:2),
        GRanges(c("chr1:1-10", "chr1:6-15"), score1 = 3:4)
    )
    rownames <- paste0("chr1:", c(1, 6, 1, 6), "-", c(10, 15, 10, 15))
    re <- RaggedExperiment(lgr)
    expect_identical(
        assay(re),
        matrix(
            c(1:2, NA, NA, NA, NA, 3:4), ncol = 2,
            dimnames=list(rownames, NULL)
        )
    )

    ridx <- c(1, 4, 2, 3)
    expect_identical(
        assay(re[ridx,]),
        matrix(
            c(1L, NA, 2L, NA, NA, 4L, NA, 3L), ncol = 2,
            dimnames=list(rownames[ridx], NULL)
        )
    )
})

test_that("subsetByOverlaps works", {
    sample1 <- GRanges(
        c(A = "chr1:1-10:-", B = "chr1:8-14:+", C = "chr2:15-18:+"),
        score = 3:5)
    sample2 <- GRanges(
        c(D = "chr1:1-10:-", E = "chr2:11-18:+"),
        score = 1:2)
    colDat <- DataFrame(id = 1:2)
    re <- RaggedExperiment(
        sample1 = sample1,
        sample2 = sample2,
        colData = colDat)
    range <- GRanges("chr1:3-10")
    expect_identical(
        subsetByOverlaps(rowRanges(re), range),
        rowRanges(subsetByOverlaps(re, range))
    )
})

test_that("subset works", {
    sample1 <- GRanges(
        c(A = "chr1:1-10:-", B = "chr1:8-14:+", C = "chr2:15-18:+"),
        score = 3:5)
    sample2 <- GRanges(
        c(D = "chr1:1-10:-", E = "chr2:11-18:+"),
        score = 1:2)
    colDat <- DataFrame(id = 1:2)
    re <- RaggedExperiment(
        sample1 = sample1,
        sample2 = sample2,
        colData = colDat)
    res <- subset(re, subset = score >= 3L)
    expect_identical(mcols(res), DataFrame(score = 3:5))
    expect_identical(dim(res), c(3L, 2L))
    res <- subset(re, select = id == 1)
    expect_identical(colData(res), DataFrame(id = 1L, row.names = "sample1"))
    expect_identical(dim(res), c(5L, 1L))
})

re4 <- RaggedExperiment(GRangesList(
    GRanges(c(A = "chr1:1-10:-", B = "chr1:8-14:-", C = "chr2:15-18:+"),
        score = 3:5),
    GRanges(c(D = "chr1:1-10:-", E = "chr2:11-18:+"), score = 1:2)
), colData = DataFrame(id = 1:2))

query <- GRanges(c("chr1:1-14:-", "chr2:11-18:+"))

weightedmean <- function(scores, ranges, qranges)
{
    ## weighted average score per query range
    ## the weight corresponds to the size of the overlap of each
    ## overlapping subject range with the corresponding query range
    isects <- pintersect(ranges, qranges)
    sum(scores * width(isects)) / sum(width(isects))
}

qreduceAssay(re4, query, weightedmean)

\dontrun{
    ## Extended example: non-silent mutations, summarized by genic
    ## region
    suppressPackageStartupMessages({
        library(TxDb.Hsapiens.UCSC.hg19.knownGene)
        library(org.Hs.eg.db)
        library(GenomeInfoDb)
        library(MultiAssayExperiment)
        library(curatedTCGAData)
        library(TCGAutils)
    })

    ## TCGA MultiAssayExperiment with RaggedExperiment data
    mae <- curatedTCGAData("ACC", c("RNASeq2GeneNorm", "CNASNP", "Mutation"),
        version = "1.1.38", dry.run = FALSE)

    ## genomic coordinates
    gn <- genes(TxDb.Hsapiens.UCSC.hg19.knownGene)
    gn <- keepStandardChromosomes(granges(gn), pruning.mode="coarse")
    seqlevelsStyle(gn) <- "NCBI"
    genome(gn)
    gn <- unstrand(gn)

    ## reduce mutations, marking any genomic range with non-silent
    ## mutation as FALSE
    nonsilent <- function(scores, ranges, qranges)
        any(scores != "Silent")
    mre <- mae[["ACC_Mutation-20160128"]]
    seqlevelsStyle(rowRanges(mre)) <- "NCBI"
    ## hack to make genomes match
    genome(mre) <- paste0(correctBuild(unique(genome(mre)), "NCBI"), ".p13")
    mutations <- qreduceAssay(mre, gn, nonsilent, "Variant_Classification")
    genome(mre) <- correctBuild(unique(genome(mre)), "NCBI")

    ## reduce copy number
    re <- mae[["ACC_CNASNP-20160128"]]
    class(re)
    ## [1] "RaggedExperiment"
    seqlevelsStyle(re) <- "NCBI"
    genome(re) <- "GRCh37.p13"
    cn <- qreduceAssay(re, gn, weightedmean, "Segment_Mean")
    genome(re) <- "GRCh37"

    ## ALTERNATIVE
    ##
    ## TCGAutils helper function to convert RaggedExperiment objects to
    ## RangedSummarizedExperiment based on annotated gene ranges
    mae2 <- mae
    mae2[[1L]] <- re
    mae2[[2L]] <- mre
    qreduceTCGA(mae2)
}

.assay_i <-
    function(x, i)
{
    mcols <- .mcols(x)
    if (ncol(mcols) == 0L)
        stop("'length(assays(x))' is 0")
    if (is.character(i)) {
        i <- match(i, assayNames(x))
    } else if (is.logical(i)) {
        i <- which(i)
    }
    if (!BiocBaseUtils::isScalarNumber(i))
        stop("'i' must be a single number")
    if (ncol(mcols) < i)
        stop("'i' is greater than 'length(assays(x))'")
    i
}

#' @name assay-functions
#'
#' @title Create simplified representation of ragged assay data.
#'
#' @description These methods transform \code{assay()} from the
#'     default (i.e., \code{sparseAssay()}) representation to various
#'     forms of more dense representation. \code{compactAssay()}
#'     collapses identical ranges across samples into a single
#'     row. \code{disjoinAssay()} creates disjoint (non-overlapping)
#'     regions, simplifies values within each sample in a
#'     user-specified manner, and returns a matrix of disjoint regions
#'     x samples.
#'
#' @param x A \code{RaggedExperiment} object
#'
#' @param i integer(1) or character(1) name of assay to be
#'     transformed.
#'
#' @param withDimnames logical(1) include dimnames on the returned
#'     matrix. When there are no explict rownames, these are
#'     manufactured with \code{as.character(rowRanges(x))}; rownames
#'     are always manufactured for \code{compactAssay()} and
#'     \code{disjoinAssay()}.
#'
#' @param background A value (default NA) for the returned matrix after
#'     \code{*Assay} operations
#'
#' @param sparse logical(1) whether to return a
#'     \code{\link[Matrix]{sparseMatrix}} representation
#'
#' @return \code{sparseAssay()}: A matrix() with dimensions
#'     \code{dim(x)}. Elements contain the assay value for the \emph{i}th
#'     range and \emph{j}th sample. Use 'sparse=TRUE' to obtain
#'     a \code{\link[Matrix]{sparseMatrix}} assay representation.
#'
#' @export
sparseAssay <-
    function(
        x, i = 1, withDimnames = TRUE, background = NA_integer_, sparse = FALSE
    )
{
    i <- .assay_i(x, i)
    mcol <- .mcols(x)[[i]]
    dim <- .dim(x)
    if (withDimnames) {
        dimnames <- .dimnames(x)
        if (is.null(dimnames[[1]]))
            dimnames[[1]] <- as.character(.rowRanges(x))
    } else {
        dimnames <- list(NULL, NULL)
    }

    idx <- cbind(
        row = seq_len(dim[[1L]]),
        col = rep(seq_len(dim[[2]]), lengths(.assays(x)))
    )
    if (sparse) {
        M <- Matrix::sparseMatrix(
            i = idx[, 1], j = idx[, 2], x = mcol, dims = dim
        )
        dimnames(M) <- dimnames
    } else {
        na <- as(background, class(mcol))
        M <- matrix(na, nrow = dim[[1]], ncol = dim[[2]], dimnames = dimnames)
        M[idx] <- mcol
        M <- M[.rowidx(x), .colidx(x), drop=FALSE]
    }
    M
}

#' @rdname assay-functions
#'
#' @return \code{compactAssay()}: Samples with identical range are placed
#'     in the same row. Non-disjoint ranges are NOT collapsed. Use
#'     'sparse=TRUE' to obtain a \code{\link[Matrix]{sparseMatrix}} assay
#'     representation.
#'
#' @export
compactAssay <-
    function(
        x, i = 1, withDimnames = TRUE, background = NA_integer_, sparse = FALSE
    )
{
    i <- .assay_i(x, i)
    mcol <- .mcols(x)[[i]][.rowidx(x)]
    dim <- .dim(x)
    dimnames <- if (withDimnames) dimnames(x) else list(NULL, NULL)

    gr <- rowRanges(x)
    ugr <- unique(gr)
    row <- match(gr, ugr)
    if (!is.null(dimnames[[1]])) {
        rev <- match(ugr, gr)
        dimnames[[1]] <- dimnames[[1]][rev]
    }

    if (withDimnames)
        dimnames <- list(as.character(ugr), .dimnames(x)[[2]])
    idx <- cbind(
        row = row,
        col = rep(seq_len(dim[[2]]), lengths(.assays(x)))[.rowidx(x)]
    )

    if (sparse) {
        M <- Matrix::sparseMatrix(
            i = idx[, 1], j = idx[, 2], x = mcol,
            dims = c(length(ugr), dim[[2]])
        )
        dimnames(M) <- dimnames
    } else {
        na <- as(background, class(mcol))
        M <- matrix(
            na, nrow=length(ugr), ncol=dim[[2]],
            dimnames=dimnames
        )
        M[idx] <- mcol
    }
    M[order(ugr), .colidx(x), drop=FALSE]
}

#' @rdname assay-functions
#'
#' @param simplifyDisjoin A \code{function} / functional operating on a
#'     \code{*List}, where the elements of the list are all within-sample
#'     assay values from ranges overlapping each disjoint range. For
#'     instance, to use the \code{simplifyDisjoin=mean} of overlapping ranges,
#'     where ranges are characterized by integer-valued scores, the
#'     entries are calculated as
#'     \preformatted{
#'                     a
#'     original: |-----------|
#'                         b
#'                    |----------|
#'
#'                 a    a, b   b
#'     disjoint: |----|------|---|
#'
#'     values <- IntegerList(a, c(a, b), b)
#'     simplifyDisjoin(values)
#'     }
#' @param simplifyReduce A \code{function} / functional accepting arguments
#'     \code{score}, \code{range}, and \code{qrange}:
#'
#'     \itemize{
#'
#'         \item{\code{score}} A \code{*List}, where each list element
#'             corresponds to a cell in the matrix to be returned by
#'             \code{qreduceAssay}. Vector elements correspond to
#'             ranges overlapping query. The \code{*List} objects
#'             support many vectorized mathematical operations, so
#'             \code{simplifyReduce} can be implemented efficiently.
#'
#'         \item{\code{range}} A \code{GRangesList} instance,
#'             'parallel' to \code{score}. Each element of the list
#'             corresponds to a cell in the matrix to be returned by
#'             \code{qreduceAssay}. Each range in the element
#'             corresponds to the range for which the \code{score}
#'             element applies.
#'
#'         \item{\code{qrange}} A \code{GRanges} instance with the
#'              same length as \code{unlist(score)}, providing the
#'              query range window to which the corresponding scores
#'              apply.
#'
#'     }
#' @return \code{disjoinAssay()}: A matrix with number of rows equal
#'     to number of disjoint ranges across all samples. Elements of
#'     the matrix are summarized by applying \code{simplifyDisjoin()} to
#'     assay values of overlapping ranges
#'
#' @export
disjoinAssay <- function(x, simplifyDisjoin, i = 1, withDimnames = TRUE,
        background = NA_integer_)
{
    stopifnot_simplify_ok(simplifyDisjoin, nargs=1L)
    i <- .assay_i(x, i)
    mcol <- .mcols(x)[[i]][.rowidx(x)]
    dim <- .dim(x)

    dj <- disjoin(rowRanges(x), with.revmap=TRUE)
    map <- dj$revmap

    mcol_idx <- unlist(map)
    row <- rep(seq_along(map), lengths(map))
    col <- rep(seq_len(dim[[2]]), lengths(.assays(x)))[.rowidx(x)][mcol_idx]
    group <- (row - 1L) * max(col) + col
    group <- match(group, unique(group)) # 'sorted'
    result <- simplifyDisjoin(splitAsList(mcol[mcol_idx], group))
    group <- !duplicated(group)

    na <- as(background, class(mcol))
    if (withDimnames) {
        dimnames <- list(as.character(dj), .dimnames(x)[[2]])
    } else {
        dimnames <- list(NULL, NULL)
    }

    m <- matrix(
        na, nrow=length(dj), ncol=dim[[2]],
        dimnames=dimnames
    )
    idx <- cbind(row = row[group], col = col[group])
    m[idx] <- result
    m[order(dj), .colidx(x), drop=FALSE]
}

#' @rdname assay-functions
#'
#' @title Create a reduced representation of ragged assay data.
#'
#' @description This method transforms \code{assay()} from the default
#'     (i.e., \code{sparseAssay()}) representation to a reduced
#'     representation summarizing each original range overlapping
#'     ranges in \code{query}. Reduction in each cell can be tailored
#'     to indivdual needs using the \code{simplifyReduce} functional argument.
#'
#' @param query \code{GRanges} providing regions over which reduction
#'     is to occur.
#'
#' @return \code{qreduceAssay()}: A matrix() with dimensions
#'     \code{length(query) x ncol(x)}. Elements contain assay
#'     values for the ith query range and jth sample, summarized
#'     according to the function \code{simplifyReduce}.
#'
#' @example inst/scripts/assay-functions-Ex.R
#'
#' @import GenomicRanges
#' @export
qreduceAssay <-
    function(x, query, simplifyReduce, i = 1, withDimnames = TRUE,
        background = NA_integer_)
{
    if (missing(i) && ncol(.mcols(x)) == 0)
        return(matrix(NA, 0, 0))
    stopifnot_simplify_ok(simplifyReduce, 3L)
    i <- .assay_i(x, i)

    mcol <- .mcols(x)[[i]][.rowidx(x)]
    dim <- .dim(x)
    subject <- unname(rowRanges(x))
    query <- granges(query)

    olap <- findOverlaps(query, subject)
    sidx <- subjectHits(olap)

    row <- queryHits(olap)
    col <- rep(seq_len(dim[[2]]), lengths(.assays(x)))[.rowidx(x)][sidx]
    score <- mcol[sidx]
    subject <- subject[sidx]
    qranges <- query[row]
    ranges <- restrict(
        subject,
        ## clamp subject ranges to lie within query
        start=pmax(start(qranges), start(subject)),
        end=pmin(end(qranges), end(subject))
    )

    group <- (row - 1L) * max(col, 0) + col # 'max(col, 0)' for 0-length col
    group <- match(group, unique(group)) # 'sorted'

    ugroup <- !duplicated(group)
    result <- simplifyReduce(
        unname(splitAsList(score, group)),
        unname(splitAsList(ranges, group)),
        unname(qranges)[ugroup]
    )
    group <- ugroup

    na <- as(background, class(result))
    dimnames <- list(NULL, NULL)
    if (withDimnames)
        dimnames <- list(as.character(query), .dimnames(x)[[2]])
    m <- matrix(na, nrow=length(query), ncol=dim[[2]], dimnames=dimnames)
    idx <- cbind(row = row[group], col = col[group])
    m[idx] <- result
    m[, .colidx(x), drop=FALSE]
}

context("coerce-functions")

test_that("SummarizedExperiment coercion works", {
    fun1 <- function(score, range, qrange)
        lengths(score)

    colData <-  DataFrame(x=1:2, row.names=c("A", "B"))
    reA <- RaggedExperiment(
        A=GRanges(c("A:1-10", "A:6-15"), score=1:2),
        B=GRanges("A:1-5", score=3),
        colData = colData
    )

    res <- sparseSummarizedExperiment(reA)
    expect_identical(assay(res), sparseAssay(reA))
    expect_identical(colData(res), colData(reA))
    expect_identical(assayNames(res), assayNames(reA))

    res <- compactSummarizedExperiment(reA)
    expect_identical(assay(res), compactAssay(reA))
    expect_identical(colData(res), colData(reA))

    res <- compactSummarizedExperiment(reA, withDimnames=FALSE)
    expect_identical(dimnames(assay(res)), NULL)

    res <- disjoinSummarizedExperiment(reA, simplifyDisjoin=fun1)
    expect_identical(assay(res), disjoinAssay(reA, simplifyDisjoin=fun1))
    expect_identical(colData(res), colData)

    res <- disjoinSummarizedExperiment(reA, simplifyDisjoin=fun1, withDimnames=FALSE)
    expect_identical(dimnames(assay(res)), NULL)

    res <- qreduceSummarizedExperiment(reA, simplifyReduce=fun1)
    expect_identical(
        assay(res),
        qreduceAssay(reA, rowRanges(reA), simplifyReduce=fun1)
    )
    expect_identical(colData(res), colData)

    res <- qreduceSummarizedExperiment(reA, simplifyReduce=fun1, withDimnames=FALSE)
    expect_equal(
        assay(res),
        qreduceAssay(reA, rowRanges(reA), simplifyReduce=fun1, withDimnames=FALSE),
        check.attributes = FALSE
    )
})

test_that("SummarizedExperiment coercion assay selection works", {
    fun1 <- function(score, range, qrange)
        lengths(score)

    reA <- RaggedExperiment(
        GRanges(c("A:1-10", "A:6-15"), score1=1:2, score2=5:6),
        GRanges("A:1-5", score1=3, score2=7)
    )

    res <- sparseSummarizedExperiment(reA, 2)
    expect_identical(assay(res), sparseAssay(reA, 2))

    res <- sparseSummarizedExperiment(reA, "score2")
    expect_identical(assay(res), sparseAssay(reA, "score2"))
})

test_that("sparseMatrix to RaggedExperiment coercion works", {
    sm <- Matrix::sparseMatrix(
        i = c(2, 3, 4, 3, 4, 3, 4),
        j = c(1, 1, 2, 3, 3, 4, 4),
        x = c(2L, 4L, 2L, 2L, 2L, 4L, 2L),
        dims = c(4, 4),
        dimnames = list(
            c("chr2:1-10", "chr2:2-10", "chr2:3-10", "chr2:4-10"),
            LETTERS[1:4]
        )
    )

    ragex <- as(sm, "RaggedExperiment")

    expect_true(is(ragex, "RaggedExperiment"))

    expect_identical(LETTERS[1:4], colnames(ragex))

    sm <- Matrix::sparseMatrix(
        i = c(2, 3, 4, 3, 4, 3, 4),
        j = c(1, 1, 1, 3, 3, 4, 4),
        x = c(2L, 4L, 2L, 2L, 2L, 4L, 2L),
        dims = c(4, 4),
        dimnames = list(
            c("chr2:1-10", "chr2:2-10", "chr2:3-10", "chr2:4-10"),
            LETTERS[1:4]
        )
    )

    ragex <- as(sm, "RaggedExperiment")

    expect_identical(
        LETTERS[1:4][MatrixGenerics::colSums(sm) > 0], colnames(ragex)
    )
})

context("assay-sparse")

test_that("sparseAssay() works", {
    urownames <- function(x)
        unique(as.character(SummarizedExperiment::rowRanges(x)))

    sample1 <- GRanges(c("chr1:1-10", "chr1:11-18"), score = 1:2)
    sample2 <- GRanges(c("chr1:1-10", "chr2:11-18"), score = 3:4)
    re <- RaggedExperiment(sample1, sample2)

    m0 <- matrix(
        c(1L, 2L, NA, NA, NA, NA, 3L, 4L), nrow=length(rowRanges(re)),
        dimnames = list(NULL, NULL)
    )
    expect_identical(sparseAssay(re, withDimnames = FALSE), m0)

    dimnames(m0) <- list(
        as.character(SummarizedExperiment::rowRanges(re)), NULL
    )
    expect_identical(sparseAssay(re), m0)

    ## test sparseAssay with sparseMatrix
    M0 <- Matrix::sparseMatrix(
        i = c(1, 2, 3, 4), j = c(1, 1, 2, 2), x = 1:4, dims = c(4, 2)
    )
    expect_identical(sparseAssay(re, sparse = TRUE, withDimnames = FALSE), M0)
    dimnames(M0) <- dimnames(m0)
    expect_identical(sparseAssay(re, sparse = TRUE), M0)

    ## ensure order is the same in both sparse and dense matrices
    sample1 <- GRanges(c(a = "chr1:1-10:-", b = "chr1:11-18:+"), score = 1:2)
    sample2 <- GRanges(c(c = "chr2:1-10:-", d = "chr2:11-18:+"), score = 3:4)
    colDat <- DataFrame(id = 1:2)
    re1 <- RaggedExperiment(sample1=sample1, sample2=sample2, colData = colDat)
    coords <- matrix(c(1, 1, 2, 1, 3, 2, 4, 2), ncol=2, byrow=TRUE)
    expect_equal(
        compactAssay(re1, sparse = FALSE)[coords],
        compactAssay(re1, sparse = TRUE)[coords]
    )
})


context("assay-compact")

test_that("compactAssay() works", {
    urownames <- function(x)
        unique(as.character(SummarizedExperiment::rowRanges(x)))

    sample1 <- GRanges(c("chr1:1-10", "chr1:11-18"), score = 1:2)
    sample2 <- GRanges(c("chr1:1-10", "chr2:11-18"), score = 3:4)
    re <- RaggedExperiment(sample1, sample2)

    ddimnames <- list(urownames(re), NULL)
    m0 <- matrix(
        c(1L, 2L, NA, 3L, NA, 4L), ncol=2,
        dimnames=ddimnames
    )
    expect_identical(compactAssay(re), m0)
    dimnames(m0) <- list(NULL, NULL)
    expect_identical(compactAssay(re, withDimnames=FALSE), m0)

    ## test compactAssay with sparseMatrix
    M0 <- Matrix::sparseMatrix(
        i = c(1, 2, 1, 3), j = c(1, 1, 2, 2), x = 1:4, dims = c(3, 2)
    )
    expect_identical(compactAssay(re, sparse = TRUE, withDimnames = FALSE), M0)
    dimnames(M0) <- ddimnames
    expect_identical(compactAssay(re, sparse = TRUE), M0)

    m0 <- matrix(
        c(1L, 2L, NA, 3L, NA, 4L), ncol=2,
        dimnames=ddimnames
    )
    ridx <- 3:1
    expect_identical(compactAssay(re[ridx,]), m0[1:2,])

    ridx <- c(2, 2, 1, 3)
    expect_identical(compactAssay(re[ridx,]), m0[1:2,])

    cidx <- c(1, 1, 2)
    expect_identical(compactAssay(re[,cidx]), m0[, cidx])

    expect_identical(compactAssay(re[ridx, cidx]), m0[1:2, cidx])

    ## handling non-disjoint ranges
    sample1 <- GRanges(c("chr1:1-10", "chr1:6-15"), score = 1:2)
    sample2 <- GRanges(c("chr1:6-15", "chr2:11-18"), score = 3:4)
    re <- RaggedExperiment(sample1, sample2)

    m0 <- matrix(
        c(1L, 2L, NA, NA, 3L, 4L), ncol=2,
        dimnames=list(urownames(re), NULL)
    )
    expect_identical(compactAssay(re), m0)

    ridx <- c(1, 4, 2, 3)
    expect_identical(compactAssay(re[ridx,]), m0)

    ## after subsetting
    lgr <- list(
        A=GRanges(c("chr1:1-10", "chr1:6-15"), score1 = 1:2),
        B=GRanges(c("chr1:1-10", "chr1:6-15"), score1 = 3:4)
    )
    re <- RaggedExperiment(GRangesList(lgr))[,1]
    m0 <- matrix(
        c(1L, 2L), ncol = 1,
        dimnames=list(c("chr1:1-10", "chr1:6-15"), "A")
    )
    expect_identical(compactAssay(re), m0)
})

context("assay-disjoin")

test_that("disjoinAssay() works", {
    ## no disjoint ranges
    sample1 <- GRanges(c("chr1:1-10", "chr1:21-30"), score=1:2)
    sample2 <- GRanges(c("chr1:1-10"), score=3)
    re <- RaggedExperiment(sample1, sample2)
    expect_identical(
        disjoinAssay(re, simplifyDisjoin=mean),
        matrix(
            c(1, 2, 3, NA), ncol=2,
            dimnames=list(c("chr1:1-10", "chr1:21-30"), NULL)
        )
    )

    ## withDimnames
    expect_identical(
        dimnames(disjoinAssay(re, simplifyDisjoin=mean, withDimnames=FALSE)),
        list(NULL, NULL)
    )

    ## disjoint ranges one sample
    sample1 <- GRanges(c("chr1:1-10", "chr1:6-15"), score=1:2)
    sample2 <- GRanges(c("chr1:1-10"), score=3)
    re <- RaggedExperiment(sample1, sample2)
    expect_identical(
        disjoinAssay(re, simplifyDisjoin=mean),
        matrix(
            c(1, 1.5, 2, 3, 3, NA), ncol=2,
            dimnames=list(c("chr1:1-5", "chr1:6-10", "chr1:11-15"), NULL)
        )
    )

    ## identical disjoint ranges two sample
    sample1 <- GRanges(c("chr1:1-10", "chr1:6-15"), score=1:2)
    sample2 <- GRanges(c("chr1:1-10", "chr1:6-15"), score=3:4)
    re <- RaggedExperiment(sample1, sample2)
    expect_identical(
        disjoinAssay(re, simplifyDisjoin=mean),
        matrix(
            c(1, 1.5, 2, 3, 3.5, 4), ncol=2,
            dimnames=list(c("chr1:1-5", "chr1:6-10", "chr1:11-15"), NULL)
        )
    )

    ## dissimilar disjoint ranges, two samples
    sample1 <- GRanges(c("chr1:1-10", "chr1:6-15"), score=1:2)
    sample2 <- GRanges(c("chr1:1-10", "chr1:4-12"), score=3:4)
    re <- RaggedExperiment(sample1, sample2)
    m0 <- matrix(
        c(1, 1, 1.5, 2, 2, 3, 3.5, 3.5, 4, NA), ncol=2,
        dimnames=list(
            paste0("chr1:", c(1, 4, 6, 11, 13), "-", c(3, 5, 10, 12, 15)),
            NULL
        )
    )
    expect_identical(disjoinAssay(re, simplifyDisjoin=mean), m0)

    ## shuffled columns
    cidx <- c(2, 1, 2)
    expect_identical(
        disjoinAssay(re[, cidx], simplifyDisjoin=mean),
        m0[, cidx]
    )

    ## shuffle rows
    ridx <- c(1, 4, 2, 3)
    expect_identical(disjoinAssay(re[ridx, ], simplifyDisjoin=mean), m0)

    ridx <- c(1, 4, 2)
    m0 <- matrix(
        c(1,  1, 1.5, 2, 2, NA, 4,   4, 4, NA), ncol=2,
        dimnames=list(
            paste0("chr1:", c(1, 4, 6, 11, 13), "-", c(3, 5, 10, 12, 15)),
            NULL
        )
    )
    expect_identical(disjoinAssay(re[ridx, ], simplifyDisjoin=mean), m0)

    ## after subsetting
    lgr <- list(
        A=GRanges(c("chr1:1-10", "chr1:6-15"), score1 = 1:2),
        B=GRanges(c("chr1:1-10", "chr1:6-15"), score1 = 3:4)
    )
    re <- RaggedExperiment(GRangesList(lgr))[,1]
    m0 <- matrix(
        c(1, 1.5, 2), ncol = 1,
        dimnames=list(c("chr1:1-5", "chr1:6-10", "chr1:11-15"), "A")
    )
    expect_identical(disjoinAssay(re, mean), m0)
})

context("assay-qreduce")

test_that("qreduceAssay() null and simple reduction", {
    fun0 <- function(score, range, qrange)
        integer(length(score))

    expect_identical(qreduceAssay(RaggedExperiment()), matrix(NA, 0, 0))

    re <- RaggedExperiment(GRanges(score=numeric()))
    query <- GRanges()
    result <- qreduceAssay(re, query, fun0)
    expect_equal(dim(result), c(length(query), ncol(re)))
    expect_equal(typeof(result), "integer")

    re <- RaggedExperiment(GRanges("A:1-5", score=1))
    query <- GRanges()
    result <- qreduceAssay(re, query, fun0)
    expect_equal(dim(result), c(length(query), ncol(re)))
    expect_equal(typeof(result), "integer")

    query <- GRanges("A:1-5")
    result <- qreduceAssay(re, query, fun0)
    exp <- matrix(
        0L, length(query), ncol(re),
        dimnames=list(as.character(query), colnames(re))
    )
    expect_identical(result, exp)

    query <- GRanges("A:10-15")
    result <- qreduceAssay(re, query, fun0)
    exp <- matrix(
        NA_integer_, length(query), ncol(re),
        dimnames=list(as.character(query), colnames(re))
    )
    expect_identical(result, exp)

    query <- GRanges(c("A:1-2", "A:3-4"))
    result <- qreduceAssay(re, query, fun0)
    exp <- matrix(
        0L, length(query), ncol(re),
        dimnames=list(as.character(query), colnames(re))
    )
    expect_identical(result, exp)

    query <- GRanges(c("A:1-2", "A:6-8"))
    result <- qreduceAssay(re, query, fun0)
    exp <- matrix(
        c(0L, NA), length(query), ncol(re),
        dimnames=list(as.character(query), colnames(re))
    )
    expect_identical(result, exp)

})

test_that("qreduceAssay() disjoint ranges works", {
    fun1 <- function(score, range, qrange)
        lengths(score)

    re <- RaggedExperiment(
        GRanges(c("A:1-10", "A:6-15"), score=1:2),
        GRanges("A:1-5", score=3)
    )

    query <- GRanges("A:1-10")
    result <- qreduceAssay(re, query, fun1)
    exp <- matrix(
        c(2L, 1L), length(query),
        dimnames=list(as.character(query), colnames(re))
    )
    expect_identical(result, exp)

    query <- GRanges(c("A:1-5", "A:6-10", "A:11-15", "A:16-20"))
    result <- qreduceAssay(re, query, fun1)
    exp <- matrix(
        c(1L, 2L, 1L, NA, 1L, NA, NA, NA), length(query),
        dimnames=list(as.character(query), colnames(re))
    )
    expect_identical(result, exp)

    ## after subsetting
    lgr <- list(
        A=GRanges(c("A:1-10", "A:6-15"), score1 = 1:2),
        B=GRanges(c("A:1-10", "A:6-15"), score1 = 3:4)
    )
    re <- RaggedExperiment(GRangesList(lgr))[,1]
    m0 <- matrix(
        c(1L, 2L, 1L, NA), ncol = 1,
        dimnames=list(c("A:1-5", "A:6-10", "A:11-15", "A:16-20"), "A")
    )
    expect_identical(qreduceAssay(re[,1], query, fun1), m0)
})

test_that("qreduceAssay() on disjoint subset works", {
    fun1 <- function(score, range, qrange)
        lengths(score)

    re <- RaggedExperiment(
        GRanges(c("A:1-10", "A:6-15"), score=1:2),
        GRanges("A:1-5", score=3)
    )

    ridx <- c(1, 1, 2, 3)
    query <- GRanges(c("A:1-5", "A:6-10", "A:11-15", "A:16-20"))
    result <- qreduceAssay(re[ridx,], query, fun1)
    exp <- matrix(
        c(2L, 3L, 1L, NA, 1L, NA, NA, NA), length(query),
        dimnames=list(as.character(query), colnames(re))
    )
    expect_identical(result, exp)

    cidx <- c(2, 2, 1)
    result <- qreduceAssay(re[ridx, cidx], query, fun1)
    expect_identical(result, exp[, cidx])
})

x <- RaggedExperiment(GRangesList(
    GRanges(c("A:1-5", "A:4-6", "A:10-15"), score=1:3),
    GRanges(c("A:1-5", "B:1-3"), score=4:5)
))

## sparseSummarizedExperiment

sse <- sparseSummarizedExperiment(x)
assay(sse)
rowRanges(sse)

## compactSummarizedExperiment

cse <- compactSummarizedExperiment(x)
assay(cse)
rowRanges(cse)

## disjoinSummarizedExperiment

disjoinAssay(x, lengths)
dse <- disjoinSummarizedExperiment(x, lengths)
assay(dse)
rowRanges(dse)

## qreduceSummarizedExperiment

x <- RaggedExperiment(GRangesList(
    GRanges(c("A:1-3", "A:4-5", "A:10-15"), score=1:3),
    GRanges(c("A:4-5", "B:1-3"), score=4:5)
))
query <- GRanges(c("A:1-2", "A:4-5", "B:1-5"))

weightedmean <- function(scores, ranges, qranges)
{
    ## weighted average score per query range
    ## the weight corresponds to the size of the overlap of each
    ## overlapping subject range with the corresponding query range
    isects <- pintersect(ranges, qranges)
    sum(scores * width(isects)) / sum(width(isects))
}

qreduceAssay(x, query, weightedmean)
qse <- qreduceSummarizedExperiment(x, query, weightedmean)
assay(qse)
rowRanges(qse)

## Create an empty RaggedExperiment instance
re0 <- RaggedExperiment()
re0

## Create a couple of GRanges objects with row ranges names
sample1 <- GRanges(
    c(a = "chr1:1-10:-", b = "chr1:11-18:+"),
    score = 1:2)
sample2 <- GRanges(
    c(c = "chr2:1-10:-", d = "chr2:11-18:+"),
    score = 3:4)

## Include column data
colDat <- DataFrame(id = 1:2)

## Create a RaggedExperiment object from a couple of GRanges
re1 <- RaggedExperiment(sample1=sample1, sample2=sample2, colData = colDat)
re1

## With list of GRanges
lgr <- list(sample1 = sample1, sample2 = sample2)

## Create a RaggedExperiment from a list of GRanges
re2 <- RaggedExperiment(lgr, colData = colDat)

grl <- GRangesList(sample1 = sample1, sample2 = sample2)

## Create a RaggedExperiment from a GRangesList
re3 <- RaggedExperiment(grl, colData = colDat)

## Subset a RaggedExperiment
assay(re3[c(1, 3),])
subsetByOverlaps(re3, GRanges("chr1:1-5"))  # by ranges

#' RaggedExperiment: Range-based data representation package
#'
#' @description \link{RaggedExperiment} allows the user to represent, copy
#'     number, mutation, and other types of range-based data formats
#'     where optional information about samples can be provided. At
#'     the backbone of this package is the \link[GenomicRanges]{GRangesList}
#'     class. The RaggedExperiment class uses this representation
#'     and presents the data in a couple of different ways:
#'     \itemize{
#'     \item rowRanges
#'     \item colData
#'     }
#'     The \link[RaggedExperiment:RaggedExperiment-class]{rowRanges}
#'     method will return the internal \code{GRangesList} representation of the
#'     dataset. A distinction between the
#'     \link[SummarizedExperiment]{SummarizedExperiment} and the
#'     \code{RaggedExperiment} classes is that the \link{RaggedExperiment} class
#'     allows for ragged ranges, meaning that there may be a different number of
#'     ranges or rows per sample.
#' @aliases RaggedExperiment-package NULL
"_PACKAGE"

#' @rdname coerce-functions
#'
#' @title Create SummarizedExperiment representations by transforming
#'     ragged assays to rectangular form.
#'
#' @description These methods transform \code{RaggedExperiment}
#'     objects to similar \code{SummarizedExperiment} objects. They do
#'     so by transforming assay data to more rectangular
#'     representations, following the rules outlined for similarly
#'     names transformations \code{sparseAssay()},
#'     \code{compactAssay()}, \code{disjoinAssay()}, and
#'     \code{qreduceAssay()}. Because of the complexity of the
#'     transformation, ti usually only makes sense transform
#'     \code{RaggedExperiment} objects with a single assay; this is
#'     currently enforced at time of coercion.
#'
#' @param x \code{RaggedExperiment}
#'
#' @param i \code{integer(1)}, \code{character(1)}, or
#'     \code{logical()} selecting the assay to be transformed.
#'
#' @param simplifyDisjoin \code{function} of 1 argument, used to
#'     transform assays. See \code{\link{assay-functions}}.
#'
#' @param simplifyReduce \code{function} of 3 arguments used to transform
#'     assays. See \code{\link{assay-functions}}.
#'
#' @param query \code{GRanges} provding regions over which reduction
#'     is to occur.
#'
#' @param withDimnames \code{logical(1)} default TRUE. propagate
#'     dimnames to SummarizedExperiment.
#'
#' @param sparse logical(1) whether to return a
#'     \code{\link[Matrix]{sparseMatrix}} representation
#'
#' @return All functions return \code{RangedSummarizedExperiment}.
#'
#' @return \code{sparseSummarizedExperiment} has \code{rowRanges()}
#'     identical to the row ranges of \code{x}, and \code{assay()}
#'     data as \code{sparseAssay()}. This is very space-inefficient
#'     representation of ragged data. Use 'sparse=TRUE' to obtain
#'     a \code{\link[Matrix]{sparseMatrix}} assay representation.
#'
#' @example inst/scripts/coerce-functions-Ex.R
#'
#' @importFrom SummarizedExperiment SummarizedExperiment rowData
#'
#' @export
sparseSummarizedExperiment <-
    function(x, i = 1, withDimnames=TRUE, sparse = FALSE)
{
    i <- .assay_i(x, i)
    name <- assayNames(x)[[i]]
    assay <- sparseAssay(x, i, withDimnames=withDimnames, sparse = sparse)
    assay <- setNames(list(assay), name)

    colData <- colData(x)
    rowRanges <- rowRanges(x)
    if (!withDimnames) {
        names(rowRanges) <- rownames(colData) <- NULL
    }

    SummarizedExperiment(assay, rowRanges=rowRanges, colData=colData)
}

#' @rdname coerce-functions
#'
#' @return \code{compactSummarizedExperiment} has \code{rowRanges()}
#'     identical to the row ranges of \code{x}, and \code{assay()}
#'     data as \code{compactAssay()}. This is space-inefficient
#'     representation of ragged data when samples are primarily
#'     composed of different ranges. Use 'sparse=TRUE' to obtain
#'     a \code{\link[Matrix]{sparseMatrix}} assay representation.
#'
#' @importFrom GenomicRanges GRanges
#'
#' @export
compactSummarizedExperiment <-
    function(x, i = 1L, withDimnames=TRUE, sparse = FALSE)
{
    i <- .assay_i(x, i)
    name <- assayNames(x)[[i]]
    assay <- compactAssay(x, i, sparse = sparse)
    rowRanges <- setNames(GRanges(rownames(assay)), rownames(assay))
    if (!withDimnames)
        assay <- unname(assay)
    assay <- setNames(list(assay), name)

    colData <- colData(x)
    if (!withDimnames) {
        names(rowRanges) <- rownames(colData) <- NULL
    }

    SummarizedExperiment(assay, rowRanges=rowRanges, colData=colData)
}

#' @rdname coerce-functions
#'
#' @return \code{disjoinSummarizedExperiment} has \code{rowRanges()}
#'     identical to the disjoint row ranges of \code{x},
#'     \code{disjoint(rowRanges(x))}, and \code{assay()} data as
#'     \code{disjoinAssay()}.
#'
#' @export
disjoinSummarizedExperiment <-
    function(x, simplifyDisjoin, i = 1L, withDimnames=TRUE)
{
    stopifnot_simplify_ok(simplifyDisjoin, 1L)

    i <- .assay_i(x, i)
    name <- assayNames(x)[[i]]
    assay <- disjoinAssay(x, simplifyDisjoin, i)
    rowRanges <- setNames(GRanges(rownames(assay)), rownames(assay))
    if (!withDimnames)
        assay <- unname(assay)
    assay <- setNames(list(assay), name)

    colData <- colData(x)
    if (!withDimnames) {
        names(rowRanges) <- rownames(colData) <- NULL
    }

    SummarizedExperiment(assay, rowRanges=rowRanges, colData=colData)
}

#' @rdname coerce-functions
#'
#' @return \code{qreduceSummarizedExperiment} has \code{rowRanges()}
#'     identical to \code{query}, and \code{assay()} data as
#'     \code{qreduceAssay()}.
#'
#' @export
qreduceSummarizedExperiment <-
    function(x, query, simplifyReduce, i = 1L, withDimnames=TRUE)
{
    stopifnot_simplify_ok(simplifyReduce, 3L)
    if (missing(query))
        query <- rowRanges(x)

    i <- .assay_i(x, i)
    name <- assayNames(x)[[i]]
    assay <- qreduceAssay(x, query, simplifyReduce, i)
    rowRanges <- setNames(GRanges(rownames(assay)), rownames(assay))
    if (!withDimnames)
        assay <- unname(assay)
    assay <- setNames(list(assay), name)

    colData <- colData(x)
    if (!withDimnames) {
        names(rowRanges) <- rownames(colData) <- NULL
    }

    SummarizedExperiment(assay, rowRanges=rowRanges, colData=colData)
}


#' @rdname coerce-functions
#'
#' @name coerce-RaggedExperiment
#'
#' @aliases coerce,dgCMatrix,RaggedExperiment-method
#'
#' @section
#' sparseMatrix:
#'   Convert a `dgCMatrix` to a `RaggedExperiment` given that the rownames
#'   are coercible to `GRanges`.
#'
#' In the following example, `x` is a `dgCMatrix` from the `Matrix` package.
#' \preformatted{
#'     `as(x, "RaggedExperiment")`
#' }
#'
#' @md
#'
#' @examples
#'
#' sm <- Matrix::sparseMatrix(
#'     i = c(2, 3, 4, 3, 4, 3, 4),
#'     j = c(1, 1, 1, 3, 3, 4, 4),
#'     x = c(2L, 4L, 2L, 2L, 2L, 4L, 2L),
#'     dims = c(4, 4),
#'     dimnames = list(
#'         c("chr2:1-10", "chr2:2-10", "chr2:3-10", "chr2:4-10"),
#'         LETTERS[1:4]
#'     )
#' )
#'
#' as(sm, "RaggedExperiment")
#'
#' @exportMethod coerce

setAs("dgCMatrix", "RaggedExperiment",
    function(from) {
        dp <- diff(from@p)
        j <- rep.int(seq_along(dp), dp)
        idx <- from@i + 1
        x <- from@x

        ranges <- rownames(from)
        if (is.null(ranges) && length(x))
            stop("'rownames(x)' are NULL and not coercible to 'GRanges'")
        rgs <- try(GRanges(ranges[idx]), silent = TRUE)
        if (is(rgs, "try-error"))
            stop("'rownames(x)' are not coercible to 'GRanges'")
        mcols(rgs) <- DataFrame(counts = x)

        grls <- GRangesList(split(rgs, j))
        names(grls) <- colnames(from)[dp > 0]
        RaggedExperiment(grls)
    }
)

stopifnot_simplify_ok <-
    function(simplify, nargs)
{
    nms <- names(formals(simplify))
    if ((length(nms) < nargs) && !("..." %in% nms))
        stop("'simplify*()' must accept ", nargs, " arguments")
}        

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Subsetting
###

.bracketSubsetRE <- function(x, i, j, ..., drop = TRUE) {
    if (missing(i) && missing(j))
        return(x)

    if (missing(i)) {
        i <- .rowidx(x)
    } else {
        if (is(i, "Vector")) {
            i <- overlapsAny(x, i, ...)
        } else if (is.character(i)) {
            i <- match(i, rownames(x))
        } else if (!is.logical(i)) {
            i <- as.integer(i)
        }
        i <- .rowidx(x)[i]
        if (any(is.na(i)))
            stop("<", class(x), ">[i,] index out of bounds")
    }

    if (missing(j)) {
        j <- .colidx(x)
    } else {
        if (is.character(j)) {
            j <- match(j, colnames(x))
        } else if (!is.logical(j)) {
            j <- as.integer(j)
        }
        j <- .colidx(x)[j]
        if (any(is.na(j)))
            stop("<", class(x), ">[,j] index out of bounds")
    }

    ## update assays; reshape to drop unused rows and columns,
    ## re-index rowidx and colidx to match new geometry
    assays <- .assays(x)
    keepi <- seq_len(sum(lengths(assays))) %in% i
    keep <- relist(keepi, assays)
    keepj <- any(keep) | seq_along(keep) %in% j
    keep <- keep[keepj]
    if (!all(keepi) || !all(keepj)) {
        i <- match(i, which(keepi))
        j <- match(j, which(keepj))
        assays <- assays[keep]
    }

    initialize(x, assays = assays, rowidx = i, colidx = j)
}

#' @describeIn RaggedExperiment Subset a RaggedExperiment object
#' @param j integer(), character(), or logical() index selecting
#'     columns from RaggedExperiment
#' @param drop logical (default TRUE) whether to drop empty samples
#' @exportMethod [
setMethod("[", c("RaggedExperiment", "ANY", "ANY"), .bracketSubsetRE)

#' @describeIn RaggedExperiment Determine whether copy number ranges
#'     defined by \code{query} overlap ranges of \code{subject}.
#' @param query A RaggedExperiment instance.
#' @param subject,ranges
#'    Each of them can be an \link[IRanges]{IntegerRanges} (e.g.
#'    \link[IRanges]{IRanges}, \link[IRanges]{Views}) or
#'    \link[IRanges]{IntegerRangesList} (e.g. \link[IRanges]{IRangesList},
#'    \link[IRanges]{ViewsList}) derivative.
#'    In addition, if \code{subject} or \code{ranges} is an
#'    \link[IRanges]{IntegerRanges} object, \code{query} or \code{x} can be
#'    an integer vector to be converted to length-one ranges.
#'
#'    If \code{query} (or \code{x}) is an \link[IRanges]{IntegerRangesList}
#'    object, then \code{subject} (or \code{ranges}) must also be an
#'    \link[IRanges]{IntegerRangesList} object.
#'
#'    If both arguments are list-like objects with names, each list element
#'    from the 2nd argument is paired with the list element from the 1st
#'    argument with the matching name, if any. Otherwise, list elements are
#'    paired by position. The overlap is then computed between the pairs as
#'    described below.
#'
#'    If \code{subject} is omitted, \code{query} is queried against
#'    itself. In this case, and only this case, the \code{drop.self}
#'    and \code{drop.redundant} arguments are allowed. By default,
#'    the result will contain hits for each range against itself, and if
#'    there is a hit from A to B, there is also a hit for B to A. If
#'    \code{drop.self} is \code{TRUE}, all self matches are dropped. If
#'    \code{drop.redundant} is \code{TRUE}, only one of A->B and B->A
#'    is returned.
#' @inheritParams IRanges::overlapsAny
#' @return 'overlapsAny' returns a logical vector of length equal
#'     to the number of rows in the \code{query}; \code{TRUE} when the
#'     copy number region overlaps the \code{subject}.
#' @exportMethod overlapsAny
setMethod("overlapsAny", c("RaggedExperiment", "Vector"),
    function(query, subject, maxgap = 0L, minoverlap = 1L,
        type = c("any", "start", "end", "within", "equal"), ...)
{
    overlapsAny(rowRanges(query), subject,
        maxgap = maxgap, minoverlap = minoverlap, type = type, ...)
})

#' @describeIn RaggedExperiment Subset the RaggedExperiment to contain only
#'     copy number ranges overlapping ranges of \code{subject}.
#' @inheritParams IRanges::subsetByOverlaps
#' @return 'subsetByOverlaps' returns a RaggedExperiment containing
#'     only copy number regions overlapping \code{subject}.
#' @exportMethod subsetByOverlaps
setMethod("subsetByOverlaps", c("RaggedExperiment", "Vector"),
    function(x, ranges, maxgap = -1L, minoverlap = 0L,
        type = c("any", "start", "end", "within", "equal"), invert = FALSE, ...)
{
    o <- overlapsAny(query = x, subject = ranges, maxgap = maxgap,
        minoverlap = minoverlap, type = match.arg(type), ...)
    x[xor(o, invert),]
})

#' @describeIn RaggedExperiment subset helper function for dividing by rowData
#'   and / or colData values
#' @inheritParams base::subset
#' @export
setMethod("subset", "RaggedExperiment",
    function(x, subset, select, ...) {
        i <- S4Vectors:::evalqForSubset(subset, mcols(x), ...)
        j <- S4Vectors:::evalqForSubset(select, colData(x), ...)
        x[i, j]
    }
)

## ----installation, message = FALSE, warning = FALSE, eval = FALSE-------------
# if (!require("BiocManager"))
#     install.packages("BiocManager")
# 
# BiocManager::install("RaggedExperiment")

## ----load-packages, message = FALSE-------------------------------------------
library(RaggedExperiment)
library(GenomicRanges)

## ----load-ascat-sample1-------------------------------------------------------
ASCAT_data_S1 <- read.delim(
    system.file(
        "extdata", "ASCAT_Sample1.txt",
        package = "RaggedExperiment", mustWork = TRUE
    ),
    header = TRUE
)

unique(ASCAT_data_S1$sample)

head(ASCAT_data_S1, n = 10)

## ----load-ascat-all-samples---------------------------------------------------
ASCAT_data_All <- read.delim(
    system.file(
        "extdata", "ASCAT_All_Samples.txt",
        package = "RaggedExperiment", mustWork = TRUE
    ),
    header = TRUE
)

unique(ASCAT_data_All$sample)

head(ASCAT_data_All, n = 10)

## ----ascat-to-granges-manual--------------------------------------------------
sample1_ex1 <- GRanges(
    seqnames = Rle(paste0("chr", ASCAT_data_S1$chr)),
    ranges = IRanges(
        start = ASCAT_data_S1$startpos, end = ASCAT_data_S1$endpos
    ),
    strand = Rle(strand("*")),
    nmajor = ASCAT_data_S1$nMajor,
    nminor = ASCAT_data_S1$nMinor
)

sample1_ex1

## ----ascat-to-granges-makegrangesfromdataframe--------------------------------
sample1_ex2 <- makeGRangesFromDataFrame(
    ASCAT_data_S1[,-c(1)],
    ignore.strand=TRUE,
    seqnames.field="chr",
    start.field="startpos",
    end.field="endpos",
    keep.extra.columns=TRUE
)

sample1_ex2

## ----split-dataframe----------------------------------------------------------
sample_list <- split(
    ASCAT_data_All,
    f = ASCAT_data_All$sample
)

## ----ascat-to-grangeslist-single----------------------------------------------
sample_list_GRanges_ex1 <- makeGRangesListFromDataFrame(
    ASCAT_data_S1,
    ignore.strand=TRUE,
    seqnames.field="chr",
    start.field="startpos",
    end.field="endpos",
    keep.extra.columns=TRUE,
    split.field = "sample"
)

sample_list_GRanges_ex1

## ----ascat-to-grangeslist-multiple--------------------------------------------
sample_list_GRanges_ex2 <- makeGRangesListFromDataFrame(
    ASCAT_data_All,
    ignore.strand=TRUE,
    seqnames.field="chr",
    start.field="startpos",
    end.field="endpos",
    keep.extra.columns=TRUE,
    split.field = "sample"
)

sample_list_GRanges_ex2

## ----access-grangeslist-------------------------------------------------------
sample1_ex3 <- sample_list_GRanges_ex2[[1]]

sample1_ex3

## ----grangeslist-function-----------------------------------------------------
sample_list_GRanges_ex3 <- GRangesList(
    sample1 = sample1_ex1
)

sample_list_GRanges_ex3

## ----ragexp-from-granges-single-----------------------------------------------
colDat_1 = DataFrame(id = 1)

ragexp_1 <- RaggedExperiment(
    sample1 = sample1_ex2,
    colData = colDat_1
)

ragexp_1

## ----ragexp-from-grangeslist-single-------------------------------------------
ragexp_2 <- RaggedExperiment(
    sample_list_GRanges_ex1,
    colData = colDat_1
)

ragexp_2

## ----ragexp-from-grangeslist-multiple-----------------------------------------
colDat_3 = DataFrame(id = 1:3)

ragexp_3 <- RaggedExperiment(
    sample_list_GRanges_ex2,
    colData = colDat_3
)

ragexp_3

## ----ragexp-from-grangeslist-function-----------------------------------------
ragexp_4  <- RaggedExperiment(
    sample_list_GRanges_ex3,
    colData = colDat_1
)

ragexp_4

## ----session-info-------------------------------------------------------------
sessionInfo()

---
title: "ASCAT to RaggedExperiment"
author:
- name: Lydia King
  affiliation: University of Galway, Ireland
- name: Marcel Ramos
  affiliation: Roswell Park Comprehensive Cancer Center, Buffalo, NY
date: "`r BiocStyle::doc_date()`"
vignette: |
  %\VignetteIndexEntry{ASCAT to RaggedExperiment}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    toc_float: true
package: RaggedExperiment
---

# Introduction

The `r BiocStyle::Biocpkg("RaggedExperiment")` package provides a flexible data
representation for copy number, mutation and other ragged array schema for
genomic location data. The output of Allele-Specific Copy number Analysis of
Tumors (ASCAT) can be classed as a ragged array and contains whole genome
allele-specific copy number information for each sample in the analysis. For
more information on ASCAT and guidelines on how to generate ASCAT data please
see the ASCAT
[website](https://www.crick.ac.uk/research/labs/peter-van-loo/software) and
[github](https://github.com/VanLoo-lab/ascat). To carry out further analysis of
the ASCAT data, utilising the functionalities of `RaggedExperiment`, the ASCAT
data must undergo a number of operations to get it in the correct format for use
with `RaggedExperiment`.

# Installation

```{r installation, message = FALSE, warning = FALSE, eval = FALSE}
if (!require("BiocManager"))
    install.packages("BiocManager")

BiocManager::install("RaggedExperiment")
```

Loading the package:

```{r load-packages, message = FALSE}
library(RaggedExperiment)
library(GenomicRanges)
```

# Structure of ASCAT data

The data shown below is the output obtained from ASCAT. ASCAT takes Log R Ratio
(LRR) and B Allele Frequency (BAF) files and derives the allele-specific copy
number profiles of tumour cells, accounting for normal cell admixture and tumour
aneuploidy. It should be noted that if working with raw CEL files, the first
step is to preprocess the CEL files using the PennCNV-Affy pipeline described
[here](https://penncnv.openbioinformatics.org/en/latest/user-guide/affy/). The
PennCNV-Affy pipeline produces the LRR and BAF files used as inputs for ASCAT.

Depending on user preference, the output of ASCAT can be multiple files, each
one containing allele-specific copy number information for one of the samples
processed in an ASCAT run, or can be a single file containing allele-specific
copy number information for all samples processed in an ASCAT run.

Let's load up and have a look at ASCAT data that contains copy number
information for just one sample i.e. sample1. Here we load up the data, check
that it only contains allele-specific copy number calls for 1 sample and look at
the first 10 rows of the dataframe.

```{r load-ascat-sample1}
ASCAT_data_S1 <- read.delim(
    system.file(
        "extdata", "ASCAT_Sample1.txt",
        package = "RaggedExperiment", mustWork = TRUE
    ),
    header = TRUE
)

unique(ASCAT_data_S1$sample)

head(ASCAT_data_S1, n = 10)
```

Now let's load up and have a look at ASCAT data that contains copy number
information for the three processed samples i.e. sample1, sample2 and sample3.
Here we load up the data, check that it contains allele-specific copy number
calls for the 3 samples and look at the first 10 rows of the dataframe. We also
note that as expected the copy number calls for sample1 are the same as above.

```{r load-ascat-all-samples}
ASCAT_data_All <- read.delim(
    system.file(
        "extdata", "ASCAT_All_Samples.txt",
        package = "RaggedExperiment", mustWork = TRUE
    ),
    header = TRUE
)

unique(ASCAT_data_All$sample)

head(ASCAT_data_All, n = 10)
```

From the output above we can see that the ASCAT data has 6 columns named sample,
chr, startpos, endpos, nMajor and nMinor. These correspond to the sample ID,
chromosome, the start position and end position of the genomic ranges and the
copy number of the major and minor alleles i.e. the homologous chromosomes.

# Converting ASCAT data to `GRanges` format

The `RaggedExperiment` class derives from a `GRangesList` representation and can
take a `GRanges` object, a `GRangesList` or a list of `Granges` as inputs. To be
able to use the ASCAT data in `RaggedExperiment` we must convert the ASCAT data
into `GRanges` format. Ideally, we want each of our `GRanges` objects to
correspond to an individual sample.

## ASCAT to `GRanges` objects

In the case where the ASCAT data has only 1 sample it is relatively simple to
produce a `GRanges` object.

```{r ascat-to-granges-manual}
sample1_ex1 <- GRanges(
    seqnames = Rle(paste0("chr", ASCAT_data_S1$chr)),
    ranges = IRanges(
        start = ASCAT_data_S1$startpos, end = ASCAT_data_S1$endpos
    ),
    strand = Rle(strand("*")),
    nmajor = ASCAT_data_S1$nMajor,
    nminor = ASCAT_data_S1$nMinor
)

sample1_ex1
```

Here we create a `GRanges` object by taking each column of the ASCAT data and
assigning them to the appropriate argument in the `GRanges` function. From above
we can see that the chromosome information is prefixed with "chr" and becomes
the seqnames column, the start and end positions are combined into an `IRanges`
object and given to the ranges argument, the strand column contains a `*` for
each entry as we don't have strand information and the metadata columns contain
the allele-specific copy number calls and are called nmajor and nminor. The
`GRanges` object we have just created contains 41 ranges (rows) and 2 metadata
columns.

Another way that we can easily convert our ASCAT data, containing 1 sample, to a
`GRanges` object is to use the `makeGRangesFromDataFrame` function from the
`GenomicsRanges` package. Here we indicate what columns in our data correspond
to the chromosome (given to the `seqnames` argument), start and end positions
(`start.field` and `end.field` arguments), whether to ignore strand information
and assign all entries `*` (`ignore.strand`) and also whether to keep the other
columns in the dataframe, nmajor and nminor, as metadata columns
(`keep.extra.columns`).

```{r ascat-to-granges-makegrangesfromdataframe}
sample1_ex2 <- makeGRangesFromDataFrame(
    ASCAT_data_S1[,-c(1)],
    ignore.strand=TRUE,
    seqnames.field="chr",
    start.field="startpos",
    end.field="endpos",
    keep.extra.columns=TRUE
)

sample1_ex2
```

In the case where the ASCAT data contains more than 1 sample you can first use
the `split` function to split the whole dataframe into multiple dataframes, one
for each sample, and then create a `GRanges` object for each dataframe. Code to
split the dataframe, based on sample ID, is given below and then the same
procedure used to produce `sample1_ex2` can be implemented to create the
`GRanges` object. Alternatively, an easier and more efficient way to do this is
to use the `makeGRangesListFromDataFrame` function from the `GenomicsRanges`
package. This will be covered in the next section.

```{r split-dataframe}
sample_list <- split(
    ASCAT_data_All,
    f = ASCAT_data_All$sample
)
```

## ASCAT to `GRangesList` instance

To produce a `GRangesList` instance from the ASCAT dataframe we can use the
`makeGRangesListFromDataFrame` function. This function takes the same arguments
as the `makeGRangesFromDataFrame` function used above, but also has an argument
specifying how the rows of the `df` are split (`split.field`). Here we will
split on sample. This function can be used in cases where the ASCAT data
contains only 1 sample or where it contains multiple samples.

Using `makeGRangesListFromDataFrame` to create a list of `GRanges` objects where
ASCAT data has only 1 sample:

```{r ascat-to-grangeslist-single}
sample_list_GRanges_ex1 <- makeGRangesListFromDataFrame(
    ASCAT_data_S1,
    ignore.strand=TRUE,
    seqnames.field="chr",
    start.field="startpos",
    end.field="endpos",
    keep.extra.columns=TRUE,
    split.field = "sample"
)

sample_list_GRanges_ex1
```

Using `makeGRangesListFromDataFrame` to create a `list` of `GRanges` objects
where ASCAT data has multiple samples:

```{r ascat-to-grangeslist-multiple}
sample_list_GRanges_ex2 <- makeGRangesListFromDataFrame(
    ASCAT_data_All,
    ignore.strand=TRUE,
    seqnames.field="chr",
    start.field="startpos",
    end.field="endpos",
    keep.extra.columns=TRUE,
    split.field = "sample"
)

sample_list_GRanges_ex2
```

Each `GRanges` object in the `list` can then be accessed using square bracket
notation.

```{r access-grangeslist}
sample1_ex3 <- sample_list_GRanges_ex2[[1]]

sample1_ex3
```

Another way we can produce a `GRangesList` instance is to use the `GRangesList`
function. This function creates a list that contains all our `GRanges` objects.
This is straightforward in that we use the `GRangesList` function with our
`GRanges` objects as named or unnamed inputs. Below we have created a list that
includes 1 `GRanges` objects, created in section 4.1., corresponding to sample1.

```{r grangeslist-function}
sample_list_GRanges_ex3 <- GRangesList(
    sample1 = sample1_ex1
)

sample_list_GRanges_ex3
```

# Constructing a `RaggedExperiment` object from ASCAT output

Now we have created the `GRanges` objects and `GRangesList` instances we can
easily use `RaggedExperiment`.

## Using `GRanges` objects

From above we have a `GRanges` object derived from the ASCAT data containing 1
sample i.e. `sample1_ex1` / `sample1_ex2` and the capabilities to produce
individual `GRanges` objects derived from the ASCAT data containing 3 samples.
We can now use these `GRanges` objects as inputs to `RaggedExperiment`. Note
that we create column data `colData` to describe the samples.

Using `GRanges` object where ASCAT data only has 1 sample:

```{r ragexp-from-granges-single}
colDat_1 = DataFrame(id = 1)

ragexp_1 <- RaggedExperiment(
    sample1 = sample1_ex2,
    colData = colDat_1
)

ragexp_1
```

In the case where you have multiple `GRanges` objects, corresponding to
different samples, the code is similar to above. Each sample is inputted into
the `RaggedExperiment` function and `colDat_1` corresponds to the id for each
sample i.e. 1, 2 and 3, if 3 samples are provided.

## Using a `GRangesList` instance

From before we have a `GRangesList` derived from the ASCAT data containing 1
sample i.e. `sample_list_GRanges_ex1` and the `GRangesList` derived from the
ASCAT data containing 3 samples i.e. `sample_list_GRanges_ex2`. We can now use
this `GRangesList` as the input to `RaggedExperiment`.

Using `GRangesList` where ASCAT data only has 1 sample:

```{r ragexp-from-grangeslist-single}
ragexp_2 <- RaggedExperiment(
    sample_list_GRanges_ex1,
    colData = colDat_1
)

ragexp_2
```

Using `GRangesList` where ASCAT data only has multiple samples:

```{r ragexp-from-grangeslist-multiple}
colDat_3 = DataFrame(id = 1:3)

ragexp_3 <- RaggedExperiment(
    sample_list_GRanges_ex2,
    colData = colDat_3
)

ragexp_3
```

We can also use the `GRangesList` produced using the `GRangesList` function:

```{r ragexp-from-grangeslist-function}
ragexp_4  <- RaggedExperiment(
    sample_list_GRanges_ex3,
    colData = colDat_1
)

ragexp_4
```

# Downstream Analysis

Now that we have the ASCAT data converted to `RaggedExperiment` objects, we can
use the \*Assay functions that are described in the `RaggedExperiment` vignette
found in the [landing page][]. These functions provide several different
functions for representing ranged data in a rectangular matrix. They make it
easy to find genomic segments shared/not shared between each sample considered
and provide the corresponding allele-specific copy number calls for each sample
across each segment.

[landing page]: https://bioconductor.org/packages/RaggedExperiment/

# Session Information

```{r session-info}
sessionInfo()
```

---
title: "RaggedExperiment"
author:
- name: Martin Morgan
  affiliation: Roswell Park Comprehensive Cancer Center, Buffalo, NY
- name: Marcel Ramos
  affiliation: Roswell Park Comprehensive Cancer Center, Buffalo, NY
date: "`r BiocStyle::doc_date()`"
vignette: |
  %\VignetteIndexEntry{RaggedExperiment}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
output:
  BiocStyle::html_document:
    toc_float: true
package: RaggedExperiment
bibliography: ../inst/REFERENCES.bib
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(dev = 'svg')
```

# Introduction

The `r BiocStyle::Biocpkg("RaggedExperiment")` package provides a flexible data
representation for copy number, mutation and other ragged array schema for
genomic location data. It aims to provide a framework for a set of samples
that have differing numbers of genomic ranges.

The `RaggedExperiment` class derives from a `GRangesList` representation and
provides a semblance of a rectangular dataset. The row and column dimensions of
the `RaggedExperiment` correspond to the number of ranges in the entire dataset
and the number of samples represented in the data, respectively.

# Installation

```{r installation, eval = FALSE}
if (!require("BiocManager"))
    install.packages("BiocManager")
BiocManager::install("RaggedExperiment")
```

Loading the package:

```{r libs, include=TRUE,results="hide",message=FALSE,warning=FALSE}
library(RaggedExperiment)
library(GenomicRanges)
```

# Citing RaggedExperiment

Your citations are crucial in keeping our software free and open source. To
cite our package see the citation (@Ramos2023-og) in the Reference section.
You may also browse to the publication at the link [here][1].

[1]: https://doi.org/10.1093/bioinformatics/btad330


# `RaggedExperiment` class overview

A schematic showing the class geometry and supported transformations of the
`RaggedExperiment` class is show below. There are three main operations for
transforming the `RaggedExperiment` representation:

1) `sparseAssay`
2) `compactAssay`
3) `qreduceAssay`

```{r class-schematic}
#| echo = FALSE,
#| out.width = "\\maxwidth",
#| fig.cap = "RaggedExperiment object schematic. Rows and columns represent
#|   genomic ranges and samples, respectively. Assay operations can be
#|   performed with (from left to right) compactAssay, qreduceAssay, and
#|   sparseAssay."

knitr::include_graphics("../man/figures/RaggedExperiment.svg")
```

# Constructing a `RaggedExperiment` object

We start with a couple of `GRanges` objects, each representing an individual
sample:

```{r granges-objects}
sample1 <- GRanges(
    c(A = "chr1:1-10:-", B = "chr1:8-14:+", C = "chr2:15-18:+"),
    score = 3:5)
sample2 <- GRanges(
    c(D = "chr1:1-10:-", E = "chr2:11-18:+"),
    score = 1:2)
```

Include column data `colData` to describe the samples:

```{r coldata}
colDat <- DataFrame(id = 1:2)
```

## Using `GRanges` objects

```{r ragexp-from-granges}
ragexp <- RaggedExperiment(
    sample1 = sample1,
    sample2 = sample2,
    colData = colDat
)
ragexp
```

## Using a `GRangesList` instance

```{r ragexp-from-grangeslist}
grl <- GRangesList(sample1 = sample1, sample2 = sample2)
RaggedExperiment(grl, colData = colDat)
```

## Using a `list` of `GRanges`

```{r ragexp-from-list}
rangeList <- list(sample1 = sample1, sample2 = sample2)
RaggedExperiment(rangeList, colData = colDat)
```

## Using a `List` of `GRanges` with metadata

__Note__: In cases where a `SimpleGenomicRangesList` is provided along with
accompanying metadata (accessed by `mcols`), the metadata is used as the
`colData` for the `RaggedExperiment`.

```{r ragexp-from-list-mcols}
grList <- List(sample1 = sample1, sample2 = sample2)
mcols(grList) <- colDat
RaggedExperiment(grList)
```

# Accessors

## Range data

```{r accessor-rowranges}
rowRanges(ragexp)
```

## Dimension names

```{r accessor-dimnames}
dimnames(ragexp)
```

## `colData`

```{r accessor-coldata}
colData(ragexp)
```

# Subsetting

## by dimension

Subsetting a `RaggedExperiment` is akin to subsetting a `matrix` object. Rows
correspond to genomic ranges and columns to samples or specimen. It is possible
to subset using `integer`, `character`, and `logical` indices.

## by genomic ranges

The `overlapsAny` and `subsetByOverlaps` functionalities are available for use
for `RaggedExperiment`. Please see the corresponding documentation in
`RaggedExperiment` and `GenomicRanges`.

# *Assay functions

`RaggedExperiment` package provides several different functions for representing
ranged data in a rectangular matrix via the `*Assay` methods.

## sparseAssay

The most straightforward matrix representation of a `RaggedExperiment` will
return a matrix of dimensions equal to the product of the number of ranges and
samples.

```{r sparseassay-dim}
dim(ragexp)
Reduce(`*`, dim(ragexp))
sparseAssay(ragexp)
length(sparseAssay(ragexp))
```

### Support for sparse matrix output

We provide sparse matrix representations with the help of the `Matrix` package.
To obtain a sparse representation, the user can use the `sparse = TRUE`
argument.

```{r sparseassay-sparse}
sparseAssay(ragexp, sparse = TRUE)
```

This representation is of class `dgCMatrix` see the `Matrix` documentation for
more details.

## compactAssay

Samples with identical ranges are placed in the same row. Non-disjoint ranges
are **not** collapsed.

```{r compactassay}
compactAssay(ragexp)
```

Similarly, to `sparseAssay` the `compactAssay` function provides the option to
obtain a sparse matrix representation with the `sparse = TRUE` argument. This
will return a `dgCMatrix` class from the `Matrix` package.

```{r compactassay-sparse}
compactAssay(ragexp, sparse = TRUE)
```

## disjoinAssay

This function returns a matrix of disjoint ranges across all samples. Elements
of the matrix are summarized by applying the `simplifyDisjoin` functional
argument to assay values of overlapping ranges.

```{r disjoinassay}
disjoinAssay(ragexp, simplifyDisjoin = mean)
```

## qreduceAssay

The `qreduceAssay` function works with a `query` parameter that highlights
a window of ranges for the resulting matrix. The returned matrix will have
dimensions `length(query)` by `ncol(x)`. Elements contain assay values for the
_i_ th query range and the _j_ th sample, summarized according to the
`simplifyReduce` functional argument.

For demonstration purposes, we can have a look at the original `GRangesList`
and the associated scores from which the current `ragexp` object is derived:

```{r qreduceassay-unlist}
unlist(grl, use.names = FALSE)
```

This data is represented as `rowRanges` and `assays` in `RaggedExperiment`:

```{r qreduceassay-rowranges}
rowRanges(ragexp)
assay(ragexp, "score")
```

Here we provide the "query" region of interest:

```{r qreduceassay-query}
(query <- GRanges(c("chr1:1-14:-", "chr2:11-18:+")))
```

The `simplifyReduce` argument in `qreduceAssay` allows the user to summarize
overlapping regions with a custom method for the given "query" region of
interest. We provide one for calculating a weighted average score per
query range, where the weight is proportional to the overlap widths between
overlapping ranges and a query range.

_Note_ that there are three arguments to this function. See the documentation
for additional details.

```{r qreduceassay-weightedmean}
weightedmean <- function(scores, ranges, qranges)
{
    isects <- pintersect(ranges, qranges)
    sum(scores * width(isects)) / sum(width(isects))
}
```

A call to `qreduceAssay` involves the `RaggedExperiment`, the `GRanges` query
and the `simplifyReduce` functional argument.

```{r qreduceassay-call}
qreduceAssay(ragexp, query, simplifyReduce = weightedmean)
```

See the schematic for a visual representation.
<p style="text-align: right;"> <a href="#header">back to top</a> </p>

# Coercion

The `RaggedExperiment` provides a family of parallel functions for coercing to
the `SummarizedExperiment` class. By selecting a particular assay index (`i`),
a parallel assay coercion method can be achieved.

Here is the list of function names:

* `sparseSummarizedExperiment`
* `compactSummarizedExperiment`
* `disjoinSummarizedExperiment`
* `qreduceSummarizedExperiment`

See the documentation for details.

## from dgCMatrix to RaggedExperiment

In the special case where the rownames of a `sparseMatrix` are coercible to
`GRanges`, `RaggedExperiment` provides the facility to convert sparse matrices
into `RaggedExperiment`. This can be done using the `as` coercion method.  The
example below first creates an example sparse `dgCMatrix` class and then shows
the `as` method usage to this end.

```{r dgcmatrix-to-ragexp}
library(Matrix)

sm <- Matrix::sparseMatrix(
    i = c(2, 3, 4, 3, 4, 3, 4),
    j = c(1, 1, 1, 3, 3, 4, 4),
    x = c(2L, 4L, 2L, 2L, 2L, 4L, 2L),
    dims = c(4, 4),
    dimnames = list(
        c("chr2:1-10", "chr2:2-10", "chr2:3-10", "chr2:4-10"),
        LETTERS[1:4]
    )
)

as(sm, "RaggedExperiment")
```

# Session Information

```{r sessioninfo}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coerce-functions.R
\name{sparseSummarizedExperiment}
\alias{sparseSummarizedExperiment}
\alias{compactSummarizedExperiment}
\alias{disjoinSummarizedExperiment}
\alias{qreduceSummarizedExperiment}
\alias{coerce-RaggedExperiment}
\alias{coerce,dgCMatrix,RaggedExperiment-method}
\title{Create SummarizedExperiment representations by transforming
    ragged assays to rectangular form.}
\usage{
sparseSummarizedExperiment(x, i = 1, withDimnames = TRUE, sparse = FALSE)

compactSummarizedExperiment(x, i = 1L, withDimnames = TRUE, sparse = FALSE)

disjoinSummarizedExperiment(x, simplifyDisjoin, i = 1L, withDimnames = TRUE)

qreduceSummarizedExperiment(
  x,
  query,
  simplifyReduce,
  i = 1L,
  withDimnames = TRUE
)
}
\arguments{
\item{x}{\code{RaggedExperiment}}

\item{i}{\code{integer(1)}, \code{character(1)}, or
\code{logical()} selecting the assay to be transformed.}

\item{withDimnames}{\code{logical(1)} default TRUE. propagate
dimnames to SummarizedExperiment.}

\item{sparse}{logical(1) whether to return a
\code{\link[Matrix]{sparseMatrix}} representation}

\item{simplifyDisjoin}{\code{function} of 1 argument, used to
transform assays. See \code{\link{assay-functions}}.}

\item{query}{\code{GRanges} provding regions over which reduction
is to occur.}

\item{simplifyReduce}{\code{function} of 3 arguments used to transform
assays. See \code{\link{assay-functions}}.}
}
\value{
All functions return \code{RangedSummarizedExperiment}.

\code{sparseSummarizedExperiment} has \code{rowRanges()}
    identical to the row ranges of \code{x}, and \code{assay()}
    data as \code{sparseAssay()}. This is very space-inefficient
    representation of ragged data. Use 'sparse=TRUE' to obtain
    a \code{\link[Matrix]{sparseMatrix}} assay representation.

\code{compactSummarizedExperiment} has \code{rowRanges()}
    identical to the row ranges of \code{x}, and \code{assay()}
    data as \code{compactAssay()}. This is space-inefficient
    representation of ragged data when samples are primarily
    composed of different ranges. Use 'sparse=TRUE' to obtain
    a \code{\link[Matrix]{sparseMatrix}} assay representation.

\code{disjoinSummarizedExperiment} has \code{rowRanges()}
    identical to the disjoint row ranges of \code{x},
    \code{disjoint(rowRanges(x))}, and \code{assay()} data as
    \code{disjoinAssay()}.

\code{qreduceSummarizedExperiment} has \code{rowRanges()}
    identical to \code{query}, and \code{assay()} data as
    \code{qreduceAssay()}.
}
\description{
These methods transform \code{RaggedExperiment}
    objects to similar \code{SummarizedExperiment} objects. They do
    so by transforming assay data to more rectangular
    representations, following the rules outlined for similarly
    names transformations \code{sparseAssay()},
    \code{compactAssay()}, \code{disjoinAssay()}, and
    \code{qreduceAssay()}. Because of the complexity of the
    transformation, ti usually only makes sense transform
    \code{RaggedExperiment} objects with a single assay; this is
    currently enforced at time of coercion.
}
\section{sparseMatrix}{

Convert a \code{dgCMatrix} to a \code{RaggedExperiment} given that the rownames
are coercible to \code{GRanges}.

In the following example, \code{x} is a \code{dgCMatrix} from the \code{Matrix} package.
\preformatted{
    `as(x, "RaggedExperiment")`
}
}

\examples{
x <- RaggedExperiment(GRangesList(
    GRanges(c("A:1-5", "A:4-6", "A:10-15"), score=1:3),
    GRanges(c("A:1-5", "B:1-3"), score=4:5)
))

## sparseSummarizedExperiment

sse <- sparseSummarizedExperiment(x)
assay(sse)
rowRanges(sse)

## compactSummarizedExperiment

cse <- compactSummarizedExperiment(x)
assay(cse)
rowRanges(cse)

## disjoinSummarizedExperiment

disjoinAssay(x, lengths)
dse <- disjoinSummarizedExperiment(x, lengths)
assay(dse)
rowRanges(dse)

## qreduceSummarizedExperiment

x <- RaggedExperiment(GRangesList(
    GRanges(c("A:1-3", "A:4-5", "A:10-15"), score=1:3),
    GRanges(c("A:4-5", "B:1-3"), score=4:5)
))
query <- GRanges(c("A:1-2", "A:4-5", "B:1-5"))

weightedmean <- function(scores, ranges, qranges)
{
    ## weighted average score per query range
    ## the weight corresponds to the size of the overlap of each
    ## overlapping subject range with the corresponding query range
    isects <- pintersect(ranges, qranges)
    sum(scores * width(isects)) / sum(width(isects))
}

qreduceAssay(x, query, weightedmean)
qse <- qreduceSummarizedExperiment(x, query, weightedmean)
assay(qse)
rowRanges(qse)

sm <- Matrix::sparseMatrix(
    i = c(2, 3, 4, 3, 4, 3, 4),
    j = c(1, 1, 1, 3, 3, 4, 4),
    x = c(2L, 4L, 2L, 2L, 2L, 4L, 2L),
    dims = c(4, 4),
    dimnames = list(
        c("chr2:1-10", "chr2:2-10", "chr2:3-10", "chr2:4-10"),
        LETTERS[1:4]
    )
)

as(sm, "RaggedExperiment")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RaggedExperiment-class.R,
%   R/RaggedExperiment-subset-methods.R
\name{RaggedExperiment-class}
\alias{RaggedExperiment-class}
\alias{RaggedExperiment}
\alias{coerce,RaggedExperiment,GRangesList-method}
\alias{coerce,GRangesList,RaggedExperiment-method}
\alias{class:RaggedExperiment}
\alias{seqinfo,RaggedExperiment-method}
\alias{seqinfo<-,RaggedExperiment-method}
\alias{rowRanges,RaggedExperiment-method}
\alias{rowRanges<-,RaggedExperiment,GRanges-method}
\alias{mcols,RaggedExperiment-method}
\alias{mcols<-,RaggedExperiment-method}
\alias{rowData,RaggedExperiment-method}
\alias{rowData<-,RaggedExperiment-method}
\alias{dim,RaggedExperiment-method}
\alias{dimnames,RaggedExperiment-method}
\alias{dimnames<-,RaggedExperiment,list-method}
\alias{dimnames<-,RaggedExperiment,ANY-method}
\alias{length,RaggedExperiment-method}
\alias{colData,RaggedExperiment-method}
\alias{colData<-,RaggedExperiment,DataFrame-method}
\alias{assay,RaggedExperiment,missing-method}
\alias{assay,RaggedExperiment,ANY-method}
\alias{assays,RaggedExperiment-method}
\alias{assayNames,RaggedExperiment-method}
\alias{show,RaggedExperiment-method}
\alias{as.list,RaggedExperiment-method}
\alias{as.data.frame,RaggedExperiment-method}
\alias{$,RaggedExperiment-method}
\alias{[,RaggedExperiment,ANY,ANY,ANY-method}
\alias{overlapsAny,RaggedExperiment,Vector-method}
\alias{subsetByOverlaps,RaggedExperiment,Vector-method}
\alias{subset,RaggedExperiment-method}
\title{RaggedExperiment objects}
\usage{
RaggedExperiment(..., colData = DataFrame(), metadata = list())

\S4method{seqinfo}{RaggedExperiment}(x)

\S4method{seqinfo}{RaggedExperiment}(x, new2old = NULL, pruning.mode = c("error", "coarse", "fine", "tidy")) <- value

\S4method{rowRanges}{RaggedExperiment}(x, ...)

\S4method{rowRanges}{RaggedExperiment,GRanges}(x, ...) <- value

\S4method{mcols}{RaggedExperiment}(x, use.names = FALSE, ...)

\S4method{mcols}{RaggedExperiment}(x, ...) <- value

\S4method{rowData}{RaggedExperiment}(x, use.names = TRUE, ...)

\S4method{rowData}{RaggedExperiment}(x, ...) <- value

\S4method{dim}{RaggedExperiment}(x)

\S4method{dimnames}{RaggedExperiment}(x)

\S4method{dimnames}{RaggedExperiment,list}(x) <- value

\S4method{dimnames}{RaggedExperiment,ANY}(x) <- value

\S4method{length}{RaggedExperiment}(x)

\S4method{colData}{RaggedExperiment}(x, ...)

\S4method{colData}{RaggedExperiment,DataFrame}(x) <- value

\S4method{assay}{RaggedExperiment,missing}(x, i, withDimnames = TRUE, ...)

\S4method{assay}{RaggedExperiment,ANY}(x, i, withDimnames = TRUE, ...)

\S4method{assays}{RaggedExperiment}(x, withDimnames = TRUE, ...)

\S4method{assayNames}{RaggedExperiment}(x, ...)

\S4method{show}{RaggedExperiment}(object)

\S4method{as.list}{RaggedExperiment}(x, ...)

\S4method{as.data.frame}{RaggedExperiment}(x, row.names = NULL, optional = FALSE, ...)

\S4method{$}{RaggedExperiment}(x, name)

\S4method{[}{RaggedExperiment,ANY,ANY,ANY}(x, i, j, ..., drop = TRUE)

\S4method{overlapsAny}{RaggedExperiment,Vector}(
  query,
  subject,
  maxgap = 0L,
  minoverlap = 1L,
  type = c("any", "start", "end", "within", "equal"),
  ...
)

\S4method{subsetByOverlaps}{RaggedExperiment,Vector}(
  x,
  ranges,
  maxgap = -1L,
  minoverlap = 0L,
  type = c("any", "start", "end", "within", "equal"),
  invert = FALSE,
  ...
)

\S4method{subset}{RaggedExperiment}(x, subset, select, ...)
}
\arguments{
\item{...}{Constructor: GRanges, list of GRanges, or GRangesList OR
assay: Additional arguments for assay. See details for more information.}

\item{colData}{A \code{\link[S4Vectors]{DataFrame}} describing samples.
Length of rowRanges must equal the number of rows in colData}

\item{metadata}{A \code{list} to include in the metadata. Any metadata
included in the input objects are lost.}

\item{x}{A RaggedExperiment object.}

\item{new2old}{
    The \code{new2old} argument allows the user to rename, drop, add and/or
    reorder the "sequence levels" in \code{x}.

    \code{new2old} can be \code{NULL} or an integer vector with one element
    per entry in \link[Seqinfo]{Seqinfo} object \code{value} (i.e. \code{new2old} and
    \code{value} must have the same length) describing how the "new" sequence
    levels should be mapped to the "old" sequence levels, that is, how the
    entries in \code{value} should be mapped to the entries in
    \code{seqinfo(x)}.
    The values in \code{new2old} must be >= 1 and <= \code{length(seqinfo(x))}.
    \code{NA}s are allowed and indicate sequence levels that are being added.
    Old sequence levels that are not represented in \code{new2old} will be
    dropped, but this will fail if those levels are in use (e.g. if \code{x}
    is a \link[GenomicRanges]{GRanges} object with ranges defined on those
    sequence levels) unless a pruning mode is specified via the
    \code{pruning.mode} argument (see below).

    If \code{new2old=NULL}, then sequence levels can only be added to the
    existing ones, that is, \code{value} must have at least as many entries
    as \code{seqinfo(x)} (i.e. \code{length(values) >= length(seqinfo(x))})
    and also \code{seqlevels(values)[seq_len(length(seqlevels(x)))]} must be
    identical to \code{seqlevels(x)}.

    Note that most of the times it's easier to proceed in 2 steps:
    \enumerate{
      \item First align the seqlevels on the left (\code{seqlevels(x)})
            with the seqlevels on the right.
      \item Then call \code{seqinfo(x) <- value}. Because
            \code{seqlevels(x)} and \code{seqlevels(value)} now are
            identical, there's no need to specify \code{new2old}.
    }
    This 2-step approach will typically look like this:
    \preformatted{    seqlevels(x) <- seqlevels(value)  # align seqlevels
    seqinfo(x) <- seqinfo(value)  # guaranteed to work
    }
    Or, if \code{x} has seqlevels not in \code{value}, it will look
    like this:
    \preformatted{    seqlevels(x, pruning.mode="coarse") <- seqlevels(value)
    seqinfo(x) <- seqinfo(value)  # guaranteed to work
    }
    The \code{pruning.mode} argument will control what happens to \code{x}
    when some of its seqlevels get droppped. See below for more information.
  }

\item{pruning.mode}{
    When some of the seqlevels to drop from \code{x} are in use (i.e. have
    ranges on them), the ranges on these sequences need to be removed before
    the seqlevels can be dropped. We call this \emph{pruning}. The
    \code{pruning.mode} argument controls how to \emph{prune} \code{x}.
    Four pruning modes are currently defined: \code{"error"}, \code{"coarse"},
    \code{"fine"}, and \code{"tidy"}. \code{"error"} is the default. In this
    mode, no pruning is done and an error is raised. The other pruning modes
    do the following:
    \itemize{
      \item \code{"coarse"}: Remove the elements in \code{x} where the
            seqlevels to drop are in use. Typically reduces the length of
            \code{x}. Note that if \code{x} is a list-like object (e.g.
            \link[GenomicRanges]{GRangesList},
            \link[GenomicAlignments]{GAlignmentPairs}, or
            \link[GenomicAlignments]{GAlignmentsList}), then any list
            element in \code{x} where at least one of the sequence levels
            to drop is in use is \emph{fully} removed. In other words,
            when \code{pruning.mode="coarse"}, the \code{seqlevels} setter
            will keep or remove \emph{full list elements} and not try to
            change their content. This guarantees that the exact ranges
            (and their order) inside the individual list elements are
            preserved. This can be a desirable property when the list
            elements represent compound features like exons grouped by
            transcript (stored in a \link[GenomicRanges]{GRangesList}
            object as returned by
            \code{\link[GenomicFeatures]{exonsBy}( , by="tx")}),
            or paired-end or fusion reads, etc...

      \item \code{"fine"}: Supported on list-like objects only. Removes
            the ranges that are on the sequences to drop. This removal is
            done within each list element of the original object \code{x}
            and doesn't affect its length or the order of its list elements.
            In other words, the pruned object is guaranteed to be
            \emph{parallel} to the original object.

      \item \code{"tidy"}: Like the \code{"fine"} pruning above but also
            removes the list elements that become empty as the result of
            the pruning. Note that this pruning mode is particularly well
            suited on a \link[GenomicRanges]{GRangesList} object that
            contains transcripts grouped by gene, as returned by
            \code{\link[GenomicFeatures]{transcriptsBy}( , by="gene")}.
            Finally note that, as a convenience, this pruning mode is
            supported on non list-like objects (e.g.
            \link[GenomicRanges]{GRanges} or
            \link[GenomicAlignments]{GAlignments} objects) and, in this
            case, is equivalent to the \code{"coarse"} mode.
    }
    See the "B. DROP SEQLEVELS FROM A LIST-LIKE OBJECT" section in the examples
    below for an extensive illustration of these pruning modes.
  }

\item{value}{\itemize{
\item{dimnames}: A \code{list} of dimension names
\item{mcols}: A \code{\link[S4Vectors]{DataFrame}} representing the
assays
}}

\item{use.names}{(logical default FALSE) whether to propagate rownames from
the object to rownames of metadata \code{DataFrame}}

\item{i}{logical(1), integer(1), or character(1) indicating the
assay to be reported. For \code{[}, \code{i} can be any
supported \code{Vector} object, e.g., \code{GRanges}.}

\item{withDimnames}{logical (default TRUE) whether to use dimension names
in the resulting object}

\item{object}{A RaggedExperiment object.}

\item{row.names}{\code{NULL} or a character vector giving the row
    names for the data frame.  Missing values are not allowed.}

\item{optional}{logical. If \code{TRUE}, setting row names and
    converting column names (to syntactic names: see
    \code{\link[base]{make.names}}) is optional.  Note that all of \R's
    \pkg{base} package \code{as.data.frame()} methods use
    \code{optional} only for column names treatment, basically with the
    meaning of \code{\link[base]{data.frame}(*, check.names = !optional)}.
    See also the \code{make.names} argument of the \code{matrix} method.}

\item{name}{
    a literal character string or a \link[base]{name} (possibly \link[base]{backtick}
    quoted).  For extraction, this is normally (see under
    \sQuote{Environments}) partially matched to the \code{\link[base]{names}}
    of the object.
  }

\item{j}{integer(), character(), or logical() index selecting
columns from RaggedExperiment}

\item{drop}{logical (default TRUE) whether to drop empty samples}

\item{query}{A RaggedExperiment instance.}

\item{subject, ranges}{Each of them can be an \link[IRanges]{IntegerRanges} (e.g.
   \link[IRanges]{IRanges}, \link[IRanges]{Views}) or
   \link[IRanges]{IntegerRangesList} (e.g. \link[IRanges]{IRangesList},
   \link[IRanges]{ViewsList}) derivative.
   In addition, if \code{subject} or \code{ranges} is an
   \link[IRanges]{IntegerRanges} object, \code{query} or \code{x} can be
   an integer vector to be converted to length-one ranges.

   If \code{query} (or \code{x}) is an \link[IRanges]{IntegerRangesList}
   object, then \code{subject} (or \code{ranges}) must also be an
   \link[IRanges]{IntegerRangesList} object.

   If both arguments are list-like objects with names, each list element
   from the 2nd argument is paired with the list element from the 1st
   argument with the matching name, if any. Otherwise, list elements are
   paired by position. The overlap is then computed between the pairs as
   described below.

   If \code{subject} is omitted, \code{query} is queried against
   itself. In this case, and only this case, the \code{drop.self}
   and \code{drop.redundant} arguments are allowed. By default,
   the result will contain hits for each range against itself, and if
   there is a hit from A to B, there is also a hit for B to A. If
   \code{drop.self} is \code{TRUE}, all self matches are dropped. If
   \code{drop.redundant} is \code{TRUE}, only one of A->B and B->A
   is returned.}

\item{maxgap}{
    A single integer >= -1.

    If \code{type} is set to \code{"any"}, \code{maxgap} is interpreted as
    the maximum \emph{gap} that is allowed between 2 ranges for the ranges
    to be considered as overlapping. The \emph{gap} between 2 ranges
    is the number of positions that separate them. The \emph{gap} between
    2 adjacent ranges is 0. By convention when one range has its start or
    end strictly inside the other (i.e. non-disjoint ranges), the \emph{gap}
    is considered to be -1.

    If \code{type} is set to anything else, \code{maxgap} has a special
    meaning that depends on the particular \code{type}. See \code{type}
    below for more information.
  }

\item{minoverlap}{
    A single non-negative integer.

    Only ranges with a minimum of \code{minoverlap} overlapping positions
    are considered to be overlapping.

    When \code{type} is \code{"any"}, at least one of \code{maxgap} and
    \code{minoverlap} must be set to its default value.
  }

\item{type}{
    By default, any overlap is accepted. By specifying the \code{type}
    parameter, one can select for specific types of overlap. The types
    correspond to operations in Allen's Interval Algebra (see
    references). If \code{type} is \code{start} or \code{end}, the
    intervals are required to have matching starts or ends,
    respectively. Specifying \code{equal} as the type returns the
    intersection of the \code{start} and \code{end} matches. If
    \code{type} is \code{within}, the query interval must be wholly
    contained within the subject interval. Note that all matches must
    additionally satisfy the \code{minoverlap} constraint described above.

    The \code{maxgap} parameter has special meaning with the special
    overlap types. For \code{start}, \code{end}, and \code{equal},
    it specifies the maximum difference in the starts, ends or both,
    respectively. For \code{within}, it is the maximum amount by which
    the subject may be wider than the query. If \code{maxgap} is set to -1
    (the default), it's replaced internally by 0.
  }

\item{invert}{
    If \code{TRUE}, keep only the ranges in \code{x} that do \emph{not}
    overlap \code{ranges}.
  }

\item{subset}{logical expression indicating elements or rows to keep:
    missing values are taken as false.}

\item{select}{
    If \code{query} is an \link[IRanges]{IntegerRanges} derivative:
    When \code{select} is \code{"all"} (the default), the results are
    returned as a \link[S4Vectors]{Hits} object.
    Otherwise the returned value is an integer vector \emph{parallel} to
    \code{query} (i.e. same length) containing the first, last,
    or arbitrary overlapping interval in \code{subject}, with \code{NA}
    indicating intervals that did not overlap any intervals in \code{subject}.

    If \code{query} is an \link[IRanges]{IntegerRangesList} derivative:
    When \code{select} is \code{"all"} (the default), the results are
    returned as a \link[S4Vectors]{HitsList} object.
    Otherwise the returned value depends on the \code{drop} argument.
    When \code{select != "all" && !drop}, an \link[IRanges]{IntegerList} is returned,
    where each element of the result corresponds to a space in \code{query}.
    When \code{select != "all" && drop}, an integer vector is returned
    containing indices that are offset to align with the unlisted \code{query}.
  }
}
\value{
constructor returns a \code{RaggedExperiment} object

'rowRanges' returns a \code{\link[GenomicRanges]{GRanges}} object
    summarizing ranges corresponding to \code{assay()} rows.

'rowRanges<-' returns a \code{\link{RaggedExperiment}} object
    with replaced ranges

'mcols' returns a \code{\link[S4Vectors]{DataFrame}} object
    of the metadata columns

'assays' returns a \code{\link[S4Vectors]{SimpleList}}

'overlapsAny' returns a logical vector of length equal
    to the number of rows in the \code{query}; \code{TRUE} when the
    copy number region overlaps the \code{subject}.

'subsetByOverlaps' returns a RaggedExperiment containing
    only copy number regions overlapping \code{subject}.
}
\description{
The \code{RaggedExperiment} class is a container for
    storing range-based data, including but not limited to copy
    number data, and mutation data. It can store a collection of
    \code{GRanges} objects, as it is derived from the
    \code{GenomicRangesList}.
}
\section{Methods (by generic)}{
\itemize{
\item \code{seqinfo(RaggedExperiment)}: seqinfo accessor

\item \code{seqinfo(RaggedExperiment) <- value}: Replace seqinfo metadata of the ranges

\item \code{rowRanges(RaggedExperiment)}: rowRanges accessor

\item \code{rowRanges(x = RaggedExperiment) <- value}: rowRanges replacement

\item \code{mcols(RaggedExperiment)}: get the metadata columns of the ranges,
rectangular representation of the 'assays'

\item \code{mcols(RaggedExperiment) <- value}: set the metadata columns of the ranges
corresponding to the assays

\item \code{rowData(RaggedExperiment)}: get the rowData or metadata for the ranges

\item \code{rowData(RaggedExperiment) <- value}: set the rowData or metadata for the ranges

\item \code{dim(RaggedExperiment)}: get dimensions (number of sample-specific row
ranges by number of samples)

\item \code{dimnames(RaggedExperiment)}: get row (sample-specific) range names
and sample names

\item \code{dimnames(x = RaggedExperiment) <- value}: set row (sample-specific) range names
and sample names

\item \code{dimnames(x = RaggedExperiment) <- value}: set row range names and sample names to NULL

\item \code{length(RaggedExperiment)}: get the length of row vectors in the object,
similar to \link[SummarizedExperiment]{SummarizedExperiment}

\item \code{colData(RaggedExperiment)}: get column data

\item \code{colData(x = RaggedExperiment) <- value}: change the colData

\item \code{assay(x = RaggedExperiment, i = missing)}: assay missing method uses first metadata column

\item \code{assay(x = RaggedExperiment, i = ANY)}: assay numeric method.

\item \code{assays(RaggedExperiment)}: assays

\item \code{assayNames(RaggedExperiment)}: names in each assay

\item \code{show(RaggedExperiment)}: show method

\item \code{as.list(RaggedExperiment)}: Allow extraction of metadata columns as a plain
\code{list}

\item \code{as.data.frame(RaggedExperiment)}: Allow conversion to plain \code{data.frame}

\item \code{$}: Easily access the \code{colData} columns with
the dollar sign operator

\item \code{x[i}: Subset a RaggedExperiment object

\item \code{overlapsAny(query = RaggedExperiment, subject = Vector)}: Determine whether copy number ranges
defined by \code{query} overlap ranges of \code{subject}.

\item \code{subsetByOverlaps(x = RaggedExperiment, ranges = Vector)}: Subset the RaggedExperiment to contain only
copy number ranges overlapping ranges of \code{subject}.

\item \code{subset(RaggedExperiment)}: subset helper function for dividing by rowData
and / or colData values

}}
\section{Constructors}{


\code{RaggedExperiment(..., colData=DataFrame())}: Creates a
RaggedExperiment object using multiple \code{GRanges} objects or a list
of \code{GRanges} objects.  Additional column data may be provided
as a \code{DataFrame} object.
}

\section{Accessors}{


In the following, 'x' represents a \code{RaggedExperiment} object:

    \code{rowRanges(x)}:

    Get the ranged data. Value is a \code{GenomicRanges} object.

    \code{assays(x)}:

    Get the assays. Value is a \code{\link[S4Vectors]{SimpleList}}.

    \code{assay(x, i)}:

    An alternative to \code{assays(x)[[i]]} to get the \emph{i}th
    (default first) assay element.

    \code{mcols(x), mcols(x) <- value}:

    Get or set the metadata columns. For \code{RaggedExperiment}, the
    columns correspond to the assay \emph{i}th elements.

    \code{rowData(x), rowData(x) <- value}:

    Get or set the row data. Value is a \code{\link[S4Vectors]{DataFrame}}
    object. Also corresponds to the \code{mcols} data.

    \strong{\emph{Note}} for advanced users and developers. Both
    \code{mcols} and \code{rowData} setters may reduce the size of the
    internal \code{RaggedExperiment} data representation. Particularly after
    subsetting, the internal row index is modified and such setter
    operations will use the index to subset the data and reduce the
    "rows" of the internal data representation.
}

\section{Subsetting}{


   \code{x[i, j]}:
   Get ranges or elements (\code{i} and \code{j}, respectively) with
   optional metadata columns where \code{i} or \code{j} can be missing,
   an NA-free logical, numeric, or character vector.
}

\section{Coercion}{


In the following, 'object' represents a \code{RaggedExperiment} object:

\code{as(object, "GRangesList")}:

Creates a \link[GenomicRanges]{GRangesList} object from a
\code{RaggedExperiment}.

\code{as(from, "RaggedExperiment")}:

Creates a \code{RaggedExperiment} object from a
\link[GenomicRanges]{GRangesList}, or \link[GenomicRanges]{GRanges} object.
}

\examples{
## Create an empty RaggedExperiment instance
re0 <- RaggedExperiment()
re0

## Create a couple of GRanges objects with row ranges names
sample1 <- GRanges(
    c(a = "chr1:1-10:-", b = "chr1:11-18:+"),
    score = 1:2)
sample2 <- GRanges(
    c(c = "chr2:1-10:-", d = "chr2:11-18:+"),
    score = 3:4)

## Include column data
colDat <- DataFrame(id = 1:2)

## Create a RaggedExperiment object from a couple of GRanges
re1 <- RaggedExperiment(sample1=sample1, sample2=sample2, colData = colDat)
re1

## With list of GRanges
lgr <- list(sample1 = sample1, sample2 = sample2)

## Create a RaggedExperiment from a list of GRanges
re2 <- RaggedExperiment(lgr, colData = colDat)

grl <- GRangesList(sample1 = sample1, sample2 = sample2)

## Create a RaggedExperiment from a GRangesList
re3 <- RaggedExperiment(grl, colData = colDat)

## Subset a RaggedExperiment
assay(re3[c(1, 3),])
subsetByOverlaps(re3, GRanges("chr1:1-5"))  # by ranges
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RaggedExperiment-package.R
\docType{package}
\name{RaggedExperiment-package}
\alias{RaggedExperiment-package}
\title{RaggedExperiment: Range-based data representation package}
\description{
\link{RaggedExperiment} allows the user to represent, copy
    number, mutation, and other types of range-based data formats
    where optional information about samples can be provided. At
    the backbone of this package is the \link[GenomicRanges]{GRangesList}
    class. The RaggedExperiment class uses this representation
    and presents the data in a couple of different ways:
    \itemize{
    \item rowRanges
    \item colData
    }
    The \link[RaggedExperiment:RaggedExperiment-class]{rowRanges}
    method will return the internal \code{GRangesList} representation of the
    dataset. A distinction between the
    \link[SummarizedExperiment]{SummarizedExperiment} and the
    \code{RaggedExperiment} classes is that the \link{RaggedExperiment} class
    allows for ragged ranges, meaning that there may be a different number of
    ranges or rows per sample.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://bioconductor.github.io/RaggedExperiment}
  \item \url{https://bioconductor.org/packages/RaggedExperiment}
  \item Report bugs at \url{https://github.com/Bioconductor/RaggedExperiment/issues}
}

}
\author{
\strong{Maintainer}: Marcel Ramos \email{marcel.ramos@sph.cuny.edu} (\href{https://orcid.org/0000-0002-3242-0582}{ORCID})

Authors:
\itemize{
  \item Martin Morgan \email{martin.morgan@roswellpark.org}
}

Other contributors:
\itemize{
  \item Lydia King \email{L.King18@nuigalway.ie} [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assay-functions.R
\name{assay-functions}
\alias{assay-functions}
\alias{sparseAssay}
\alias{compactAssay}
\alias{disjoinAssay}
\alias{qreduceAssay}
\title{Create simplified representation of ragged assay data.}
\usage{
sparseAssay(
  x,
  i = 1,
  withDimnames = TRUE,
  background = NA_integer_,
  sparse = FALSE
)

compactAssay(
  x,
  i = 1,
  withDimnames = TRUE,
  background = NA_integer_,
  sparse = FALSE
)

disjoinAssay(
  x,
  simplifyDisjoin,
  i = 1,
  withDimnames = TRUE,
  background = NA_integer_
)

qreduceAssay(
  x,
  query,
  simplifyReduce,
  i = 1,
  withDimnames = TRUE,
  background = NA_integer_
)
}
\arguments{
\item{x}{A \code{RaggedExperiment} object}

\item{i}{integer(1) or character(1) name of assay to be
transformed.}

\item{withDimnames}{logical(1) include dimnames on the returned
matrix. When there are no explict rownames, these are
manufactured with \code{as.character(rowRanges(x))}; rownames
are always manufactured for \code{compactAssay()} and
\code{disjoinAssay()}.}

\item{background}{A value (default NA) for the returned matrix after
\code{*Assay} operations}

\item{sparse}{logical(1) whether to return a
\code{\link[Matrix]{sparseMatrix}} representation}

\item{simplifyDisjoin}{A \code{function} / functional operating on a
    \code{*List}, where the elements of the list are all within-sample
    assay values from ranges overlapping each disjoint range. For
    instance, to use the \code{simplifyDisjoin=mean} of overlapping ranges,
    where ranges are characterized by integer-valued scores, the
    entries are calculated as
    \preformatted{
                    a
    original: |-----------|
                        b
                   |----------|

                a    a, b   b
    disjoint: |----|------|---|

    values <- IntegerList(a, c(a, b), b)
    simplifyDisjoin(values)
    }}

\item{query}{\code{GRanges} providing regions over which reduction
is to occur.}

\item{simplifyReduce}{A \code{function} / functional accepting arguments
    \code{score}, \code{range}, and \code{qrange}:

    \itemize{

        \item{\code{score}} A \code{*List}, where each list element
            corresponds to a cell in the matrix to be returned by
            \code{qreduceAssay}. Vector elements correspond to
            ranges overlapping query. The \code{*List} objects
            support many vectorized mathematical operations, so
            \code{simplifyReduce} can be implemented efficiently.

        \item{\code{range}} A \code{GRangesList} instance,
            'parallel' to \code{score}. Each element of the list
            corresponds to a cell in the matrix to be returned by
            \code{qreduceAssay}. Each range in the element
            corresponds to the range for which the \code{score}
            element applies.

        \item{\code{qrange}} A \code{GRanges} instance with the
             same length as \code{unlist(score)}, providing the
             query range window to which the corresponding scores
             apply.

    }}
}
\value{
\code{sparseAssay()}: A matrix() with dimensions
    \code{dim(x)}. Elements contain the assay value for the \emph{i}th
    range and \emph{j}th sample. Use 'sparse=TRUE' to obtain
    a \code{\link[Matrix]{sparseMatrix}} assay representation.

\code{compactAssay()}: Samples with identical range are placed
    in the same row. Non-disjoint ranges are NOT collapsed. Use
    'sparse=TRUE' to obtain a \code{\link[Matrix]{sparseMatrix}} assay
    representation.

\code{disjoinAssay()}: A matrix with number of rows equal
    to number of disjoint ranges across all samples. Elements of
    the matrix are summarized by applying \code{simplifyDisjoin()} to
    assay values of overlapping ranges

\code{qreduceAssay()}: A matrix() with dimensions
    \code{length(query) x ncol(x)}. Elements contain assay
    values for the ith query range and jth sample, summarized
    according to the function \code{simplifyReduce}.
}
\description{
These methods transform \code{assay()} from the
    default (i.e., \code{sparseAssay()}) representation to various
    forms of more dense representation. \code{compactAssay()}
    collapses identical ranges across samples into a single
    row. \code{disjoinAssay()} creates disjoint (non-overlapping)
    regions, simplifies values within each sample in a
    user-specified manner, and returns a matrix of disjoint regions
    x samples.

This method transforms \code{assay()} from the default
    (i.e., \code{sparseAssay()}) representation to a reduced
    representation summarizing each original range overlapping
    ranges in \code{query}. Reduction in each cell can be tailored
    to indivdual needs using the \code{simplifyReduce} functional argument.
}
\examples{
re4 <- RaggedExperiment(GRangesList(
    GRanges(c(A = "chr1:1-10:-", B = "chr1:8-14:-", C = "chr2:15-18:+"),
        score = 3:5),
    GRanges(c(D = "chr1:1-10:-", E = "chr2:11-18:+"), score = 1:2)
), colData = DataFrame(id = 1:2))

query <- GRanges(c("chr1:1-14:-", "chr2:11-18:+"))

weightedmean <- function(scores, ranges, qranges)
{
    ## weighted average score per query range
    ## the weight corresponds to the size of the overlap of each
    ## overlapping subject range with the corresponding query range
    isects <- pintersect(ranges, qranges)
    sum(scores * width(isects)) / sum(width(isects))
}

qreduceAssay(re4, query, weightedmean)

\dontrun{
    ## Extended example: non-silent mutations, summarized by genic
    ## region
    suppressPackageStartupMessages({
        library(TxDb.Hsapiens.UCSC.hg19.knownGene)
        library(org.Hs.eg.db)
        library(GenomeInfoDb)
        library(MultiAssayExperiment)
        library(curatedTCGAData)
        library(TCGAutils)
    })

    ## TCGA MultiAssayExperiment with RaggedExperiment data
    mae <- curatedTCGAData("ACC", c("RNASeq2GeneNorm", "CNASNP", "Mutation"),
        version = "1.1.38", dry.run = FALSE)

    ## genomic coordinates
    gn <- genes(TxDb.Hsapiens.UCSC.hg19.knownGene)
    gn <- keepStandardChromosomes(granges(gn), pruning.mode="coarse")
    seqlevelsStyle(gn) <- "NCBI"
    genome(gn)
    gn <- unstrand(gn)

    ## reduce mutations, marking any genomic range with non-silent
    ## mutation as FALSE
    nonsilent <- function(scores, ranges, qranges)
        any(scores != "Silent")
    mre <- mae[["ACC_Mutation-20160128"]]
    seqlevelsStyle(rowRanges(mre)) <- "NCBI"
    ## hack to make genomes match
    genome(mre) <- paste0(correctBuild(unique(genome(mre)), "NCBI"), ".p13")
    mutations <- qreduceAssay(mre, gn, nonsilent, "Variant_Classification")
    genome(mre) <- correctBuild(unique(genome(mre)), "NCBI")

    ## reduce copy number
    re <- mae[["ACC_CNASNP-20160128"]]
    class(re)
    ## [1] "RaggedExperiment"
    seqlevelsStyle(re) <- "NCBI"
    genome(re) <- "GRCh37.p13"
    cn <- qreduceAssay(re, gn, weightedmean, "Segment_Mean")
    genome(re) <- "GRCh37"

    ## ALTERNATIVE
    ##
    ## TCGAutils helper function to convert RaggedExperiment objects to
    ## RangedSummarizedExperiment based on annotated gene ranges
    mae2 <- mae
    mae2[[1L]] <- re
    mae2[[2L]] <- mre
    qreduceTCGA(mae2)
}
}
